@@ -5,9 +5,9 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
@@ -22,6 +22,7 @@ import java.security.AccessController;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 
+import javax.servlet.AsyncContext;
 import javax.servlet.DispatcherType;
 import javax.servlet.RequestDispatcher;
 import javax.servlet.Servlet;
@@ -32,20 +33,19 @@ import javax.servlet.ServletRequestWrapper;
 import javax.servlet.ServletResponse;
 import javax.servlet.ServletResponseWrapper;
 import javax.servlet.UnavailableException;
+import javax.servlet.http.HttpServletMapping;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
 import org.apache.catalina.AsyncDispatcher;
 import org.apache.catalina.Context;
 import org.apache.catalina.Globals;
-import org.apache.catalina.InstanceEvent;
 import org.apache.catalina.Wrapper;
 import org.apache.catalina.connector.ClientAbortException;
 import org.apache.catalina.connector.Request;
 import org.apache.catalina.connector.RequestFacade;
 import org.apache.catalina.connector.Response;
 import org.apache.catalina.connector.ResponseFacade;
-import org.apache.catalina.util.InstanceSupport;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.res.StringManager;
 
@@ -63,29 +63,28 @@ import org.apache.tomcat.util.res.StringManager;
  */
 final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher {
 
-    protected static final boolean STRICT_SERVLET_COMPLIANCE;
+    static final boolean STRICT_SERVLET_COMPLIANCE;
 
-    protected static final boolean WRAP_SAME_OBJECT;
+    static final boolean WRAP_SAME_OBJECT;
 
 
     static {
         STRICT_SERVLET_COMPLIANCE = Globals.STRICT_SERVLET_COMPLIANCE;
-        
+
         String wrapSameObject = System.getProperty(
                 "org.apache.catalina.core.ApplicationDispatcher.WRAP_SAME_OBJECT");
         if (wrapSameObject == null) {
             WRAP_SAME_OBJECT = STRICT_SERVLET_COMPLIANCE;
         } else {
-            WRAP_SAME_OBJECT =
-                Boolean.valueOf(wrapSameObject).booleanValue();
+            WRAP_SAME_OBJECT = Boolean.parseBoolean(wrapSameObject);
         }
     }
 
 
     protected class PrivilegedForward
             implements PrivilegedExceptionAction<Void> {
-        private ServletRequest request;
-        private ServletResponse response;
+        private final ServletRequest request;
+        private final ServletResponse response;
 
         PrivilegedForward(ServletRequest request, ServletResponse response) {
             this.request = request;
@@ -101,8 +100,8 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
 
     protected class PrivilegedInclude implements
             PrivilegedExceptionAction<Void> {
-        private ServletRequest request;
-        private ServletResponse response;
+        private final ServletRequest request;
+        private final ServletResponse response;
 
         PrivilegedInclude(ServletRequest request, ServletResponse response) {
             this.request = request;
@@ -157,7 +156,7 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
          * The outermost response that will be passed on to the invoked servlet.
          */
         ServletResponse outerResponse = null;
-        
+
         /**
          * The request wrapper we have created and installed (if any).
          */
@@ -168,7 +167,7 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
          * The response wrapper we have created and installed (if any).
          */
         ServletResponse wrapResponse = null;
-        
+
         /**
          * Are we performing an include() instead of a forward()?
          */
@@ -202,12 +201,13 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
      *  (if any)
      * @param queryString Query string parameters included with this request
      *  (if any)
+     * @param mapping The mapping for this resource (if any)
      * @param name Servlet name (if a named dispatcher was created)
      *  else <code>null</code>
      */
     public ApplicationDispatcher
         (Wrapper wrapper, String requestURI, String servletPath,
-         String pathInfo, String queryString, String name) {
+         String pathInfo, String queryString, HttpServletMapping mapping, String name) {
 
         super();
 
@@ -218,12 +218,8 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         this.servletPath = servletPath;
         this.pathInfo = pathInfo;
         this.queryString = queryString;
+        this.mapping = mapping;
         this.name = name;
-        if (wrapper instanceof StandardWrapper)
-            this.support = ((StandardWrapper) wrapper).getInstanceSupport();
-        else
-            this.support = new InstanceSupport(wrapper);
-
     }
 
 
@@ -232,78 +228,56 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
     /**
      * The Context this RequestDispatcher is associated with.
      */
-    private Context context = null;
-
-
-    /**
-     * Descriptive information about this implementation.
-     */
-    private static final String info =
-        "org.apache.catalina.core.ApplicationDispatcher/1.0";
+    private final Context context;
 
 
     /**
      * The servlet name for a named dispatcher.
      */
-    private String name = null;
+    private final String name;
 
 
     /**
      * The extra path information for this RequestDispatcher.
      */
-    private String pathInfo = null;
+    private final String pathInfo;
 
 
     /**
      * The query string parameters for this RequestDispatcher.
      */
-    private String queryString = null;
+    private final String queryString;
 
 
     /**
      * The request URI for this RequestDispatcher.
      */
-    private String requestURI = null;
+    private final String requestURI;
 
 
     /**
      * The servlet path for this RequestDispatcher.
      */
-    private String servletPath = null;
+    private final String servletPath;
+
+
+    /**
+     * The mapping for this RequestDispatcher.
+     */
+    private final HttpServletMapping mapping;
 
 
     /**
      * The StringManager for this package.
      */
-    private static final StringManager sm =
-      StringManager.getManager(Constants.Package);
-
-
-    /**
-     * The InstanceSupport instance associated with our Wrapper (used to
-     * send "before dispatch" and "after dispatch" events.
-     */
-    private InstanceSupport support = null;
+    private static final StringManager sm = StringManager.getManager(ApplicationDispatcher.class);
 
 
     /**
      * The Wrapper associated with the resource that will be forwarded to
      * or included.
      */
-    private Wrapper wrapper = null;
-
-
-    // ------------------------------------------------------------- Properties
-
-
-    /**
-     * Return the descriptive information about this implementation.
-     */
-    public String getInfo() {
-
-        return (info);
-
-    }
+    private final Wrapper wrapper;
 
 
     // --------------------------------------------------------- Public Methods
@@ -330,8 +304,9 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
                 AccessController.doPrivileged(dp);
             } catch (PrivilegedActionException pe) {
                 Exception e = pe.getException();
-                if (e instanceof ServletException)
+                if (e instanceof ServletException) {
                     throw (ServletException) e;
+                }
                 throw (IOException) e;
             }
         } else {
@@ -342,7 +317,7 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
     private void doForward(ServletRequest request, ServletResponse response)
         throws ServletException, IOException
     {
-        
+
         // Reset any output that has been buffered, but keep headers/cookies
         if (response.isCommitted()) {
             throw new IllegalStateException
@@ -381,12 +356,9 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         // Handle an HTTP path-based forward
         else {
 
-            ApplicationHttpRequest wrequest =
-                (ApplicationHttpRequest) wrapRequest(state);
-            String contextPath = context.getPath();
+            ApplicationHttpRequest wrequest = (ApplicationHttpRequest) wrapRequest(state);
             HttpServletRequest hrequest = state.hrequest;
-            if (hrequest.getAttribute(
-                    RequestDispatcher.FORWARD_REQUEST_URI) == null) {
+            if (hrequest.getAttribute(RequestDispatcher.FORWARD_REQUEST_URI) == null) {
                 wrequest.setAttribute(RequestDispatcher.FORWARD_REQUEST_URI,
                                       hrequest.getRequestURI());
                 wrequest.setAttribute(RequestDispatcher.FORWARD_CONTEXT_PATH,
@@ -397,9 +369,10 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
                                       hrequest.getPathInfo());
                 wrequest.setAttribute(RequestDispatcher.FORWARD_QUERY_STRING,
                                       hrequest.getQueryString());
+                wrequest.setAttribute(RequestDispatcher.FORWARD_MAPPING, hrequest.getHttpServletMapping());
             }
- 
-            wrequest.setContextPath(contextPath);
+
+            wrequest.setContextPath(context.getEncodedPath());
             wrequest.setRequestURI(requestURI);
             wrequest.setServletPath(servletPath);
             wrequest.setPathInfo(pathInfo);
@@ -407,27 +380,29 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
                 wrequest.setQueryString(queryString);
                 wrequest.setQueryParams(queryString);
             }
+            wrequest.setMapping(mapping);
 
             processRequest(request,response,state);
         }
 
-        if (request.getAsyncContext() != null) {
+        if (request.isAsyncStarted()) {
             // An async request was started during the forward, don't close the
             // response as it may be written to during the async handling
             return;
         }
 
         // This is not a real close in order to support error processing
-        if (wrapper.getLogger().isDebugEnabled() )
-            wrapper.getLogger().debug(" Disabling the response for futher output");
+        if (wrapper.getLogger().isDebugEnabled() ) {
+            wrapper.getLogger().debug(" Disabling the response for further output");
+        }
 
         if  (response instanceof ResponseFacade) {
             ((ResponseFacade) response).finish();
         } else {
             // Servlet SRV.6.2.2. The Request/Response may have been wrapped
-            // and may no longer be instance of RequestFacade 
+            // and may no longer be instance of RequestFacade
             if (wrapper.getLogger().isDebugEnabled()){
-                wrapper.getLogger().debug( " The Response is vehiculed using a wrapper: " 
+                wrapper.getLogger().debug( " The Response is vehiculed using a wrapper: "
                            + response.getClass().getName() );
             }
 
@@ -439,9 +414,7 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
                 try {
                     ServletOutputStream stream = response.getOutputStream();
                     stream.close();
-                } catch (IllegalStateException f) {
-                    // Ignore
-                } catch (IOException f) {
+                } catch (IllegalStateException | IOException f) {
                     // Ignore
                 }
             } catch (IOException e) {
@@ -451,7 +424,7 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
 
     }
 
-    
+
     /**
      * Prepare the request based on the filter configuration.
      * @param request The servlet request we are processing
@@ -461,15 +434,15 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
      * @exception IOException if an input/output error occurs
      * @exception ServletException if a servlet error occurs
      */
-    private void processRequest(ServletRequest request, 
+    private void processRequest(ServletRequest request,
                                 ServletResponse response,
                                 State state)
         throws IOException, ServletException {
-                
+
         DispatcherType disInt = (DispatcherType) request.getAttribute(Globals.DISPATCHER_TYPE_ATTR);
         if (disInt != null) {
             boolean doInvoke = true;
-            
+
             if (context.getFireRequestListenersOnForwards() &&
                     !context.fireRequestInitEvent(request)) {
                 doInvoke = false;
@@ -487,15 +460,15 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
                 } else {
                     invoke(state.outerRequest, response, state);
                 }
-                
+
                 if (context.getFireRequestListenersOnForwards()) {
                     context.fireRequestDestroyEvent(request);
                 }
             }
         }
     }
-    
-    
+
+
     /**
      * Combine the servletPath and the pathInfo. If pathInfo is
      * <code>null</code> it is ignored. If servletPath is <code>null</code> then
@@ -535,8 +508,9 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
             } catch (PrivilegedActionException pe) {
                 Exception e = pe.getException();
 
-                if (e instanceof ServletException)
+                if (e instanceof ServletException) {
                     throw (ServletException) e;
+                }
                 throw (IOException) e;
             }
         } else {
@@ -554,22 +528,20 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
             // Check SRV.8.2 / SRV.14.2.5.1 compliance
             checkSameObjects(request, response);
         }
-        
+
         // Create a wrapped response to use for this request
         wrapResponse(state);
 
         // Handle an HTTP named dispatcher include
         if (name != null) {
 
-            ApplicationHttpRequest wrequest =
-                (ApplicationHttpRequest) wrapRequest(state);
+            ApplicationHttpRequest wrequest = (ApplicationHttpRequest) wrapRequest(state);
             wrequest.setAttribute(Globals.NAMED_DISPATCHER_ATTR, name);
-            if (servletPath != null)
+            if (servletPath != null) {
                 wrequest.setServletPath(servletPath);
-            wrequest.setAttribute(Globals.DISPATCHER_TYPE_ATTR,
-                    DispatcherType.INCLUDE);
-            wrequest.setAttribute(Globals.DISPATCHER_REQUEST_PATH_ATTR,
-                    getCombinedPath());
+            }
+            wrequest.setAttribute(Globals.DISPATCHER_TYPE_ATTR, DispatcherType.INCLUDE);
+            wrequest.setAttribute(Globals.DISPATCHER_REQUEST_PATH_ATTR, getCombinedPath());
             invoke(state.outerRequest, state.outerResponse, state);
         }
 
@@ -579,24 +551,26 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
             ApplicationHttpRequest wrequest =
                 (ApplicationHttpRequest) wrapRequest(state);
             String contextPath = context.getPath();
-            if (requestURI != null)
-                wrequest.setAttribute(RequestDispatcher.INCLUDE_REQUEST_URI,
-                                      requestURI);
-            if (contextPath != null)
-                wrequest.setAttribute(RequestDispatcher.INCLUDE_CONTEXT_PATH,
-                                      contextPath);
-            if (servletPath != null)
-                wrequest.setAttribute(RequestDispatcher.INCLUDE_SERVLET_PATH,
-                                      servletPath);
-            if (pathInfo != null)
-                wrequest.setAttribute(RequestDispatcher.INCLUDE_PATH_INFO,
-                                      pathInfo);
+            if (requestURI != null) {
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_REQUEST_URI, requestURI);
+            }
+            if (contextPath != null) {
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_CONTEXT_PATH, contextPath);
+            }
+            if (servletPath != null) {
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_SERVLET_PATH, servletPath);
+            }
+            if (pathInfo != null) {
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_PATH_INFO, pathInfo);
+            }
             if (queryString != null) {
-                wrequest.setAttribute(RequestDispatcher.INCLUDE_QUERY_STRING,
-                                      queryString);
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_QUERY_STRING, queryString);
                 wrequest.setQueryParams(queryString);
             }
-            
+            if (mapping != null) {
+                wrequest.setAttribute(RequestDispatcher.INCLUDE_MAPPING, mapping);
+            }
+
             wrequest.setAttribute(Globals.DISPATCHER_TYPE_ATTR,
                     DispatcherType.INCLUDE);
             wrequest.setAttribute(Globals.DISPATCHER_REQUEST_PATH_ATTR,
@@ -617,8 +591,9 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
             } catch (PrivilegedActionException pe) {
                 Exception e = pe.getException();
 
-                if (e instanceof ServletException)
+                if (e instanceof ServletException) {
                     throw (ServletException) e;
+                }
                 throw (IOException) e;
             }
         } else {
@@ -635,19 +610,14 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         // Create a wrapped response to use for this request
         wrapResponse(state);
 
-        ApplicationHttpRequest wrequest =
-            (ApplicationHttpRequest) wrapRequest(state);
+        ApplicationHttpRequest wrequest = (ApplicationHttpRequest) wrapRequest(state);
+        HttpServletRequest hrequest = state.hrequest;
 
-        if (queryString != null) {
-            wrequest.setQueryParams(queryString);
-        }
+        wrequest.setAttribute(Globals.DISPATCHER_TYPE_ATTR, DispatcherType.ASYNC);
+        wrequest.setAttribute(Globals.DISPATCHER_REQUEST_PATH_ATTR, getCombinedPath());
+        wrequest.setAttribute(AsyncContext.ASYNC_MAPPING, hrequest.getHttpServletMapping());
 
-        wrequest.setAttribute(Globals.DISPATCHER_TYPE_ATTR,
-                DispatcherType.ASYNC);
-        wrequest.setAttribute(Globals.DISPATCHER_REQUEST_PATH_ATTR,
-                getCombinedPath());
-
-        wrequest.setContextPath(context.getPath());
+        wrequest.setContextPath(context.getEncodedPath());
         wrequest.setRequestURI(requestURI);
         wrequest.setServletPath(servletPath);
         wrequest.setPathInfo(pathInfo);
@@ -655,6 +625,9 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
             wrequest.setQueryString(queryString);
             wrequest.setQueryParams(queryString);
         }
+        if (!Globals.STRICT_SERVLET_COMPLIANCE) {
+            wrequest.setMapping(mapping);
+        }
 
         invoke(state.outerRequest, state.outerResponse, state);
     }
@@ -684,14 +657,7 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         // Checking to see if the context classloader is the current context
         // classloader. If it's not, we're saving it, and setting the context
         // classloader to the Context classloader
-        ClassLoader oldCCL = Thread.currentThread().getContextClassLoader();
-        ClassLoader contextClassLoader = context.getLoader().getClassLoader();
-
-        if (oldCCL != contextClassLoader) {
-            Thread.currentThread().setContextClassLoader(contextClassLoader);
-        } else {
-            oldCCL = null;
-        }
+        ClassLoader oldCCL = context.bind(false, null);
 
         // Initialize local variables we may need
         HttpServletResponse hresponse = state.hresponse;
@@ -704,14 +670,14 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         // Check for the servlet being marked unavailable
         if (wrapper.isUnavailable()) {
             wrapper.getLogger().warn(
-                    sm.getString("applicationDispatcher.isUnavailable", 
+                    sm.getString("applicationDispatcher.isUnavailable",
                     wrapper.getName()));
             long available = wrapper.getAvailable();
-            if ((available > 0L) && (available < Long.MAX_VALUE))
+            if ((available > 0L) && (available < Long.MAX_VALUE)) {
                 hresponse.setDateHeader("Retry-After", available);
-            hresponse.sendError(HttpServletResponse.SC_SERVICE_UNAVAILABLE, sm
-                    .getString("applicationDispatcher.isUnavailable", wrapper
-                            .getName()));
+            }
+            hresponse.sendError(HttpServletResponse.SC_SERVICE_UNAVAILABLE,
+                    sm.getString("applicationDispatcher.isUnavailable", wrapper.getName()));
             unavailable = true;
         }
 
@@ -733,43 +699,30 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
                               wrapper.getName()), e);
             servlet = null;
         }
-                
+
         // Get the FilterChain Here
-        ApplicationFilterFactory factory = ApplicationFilterFactory.getInstance();
-        ApplicationFilterChain filterChain = factory.createFilterChain(request,
-                                                                wrapper,servlet);
-        
+        ApplicationFilterChain filterChain =
+                ApplicationFilterFactory.createFilterChain(request, wrapper, servlet);
+
         // Call the service() method for the allocated servlet instance
         try {
-            support.fireInstanceEvent(InstanceEvent.BEFORE_DISPATCH_EVENT,
-                                      servlet, request, response);
             // for includes/forwards
             if ((servlet != null) && (filterChain != null)) {
                filterChain.doFilter(request, response);
              }
             // Servlet Service Method is called by the FilterChain
-            support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
-                                      servlet, request, response);
         } catch (ClientAbortException e) {
-            support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
-                                      servlet, request, response);
             ioException = e;
         } catch (IOException e) {
-            support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
-                                      servlet, request, response);
             wrapper.getLogger().error(sm.getString("applicationDispatcher.serviceException",
                              wrapper.getName()), e);
             ioException = e;
         } catch (UnavailableException e) {
-            support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
-                                      servlet, request, response);
             wrapper.getLogger().error(sm.getString("applicationDispatcher.serviceException",
                              wrapper.getName()), e);
             servletException = e;
             wrapper.unavailable(e);
         } catch (ServletException e) {
-            support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
-                                      servlet, request, response);
             Throwable rootCause = StandardWrapper.getRootCause(e);
             if (!(rootCause instanceof ClientAbortException)) {
                 wrapper.getLogger().error(sm.getString("applicationDispatcher.serviceException",
@@ -777,22 +730,14 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
             }
             servletException = e;
         } catch (RuntimeException e) {
-            support.fireInstanceEvent(InstanceEvent.AFTER_DISPATCH_EVENT,
-                                      servlet, request, response);
             wrapper.getLogger().error(sm.getString("applicationDispatcher.serviceException",
                              wrapper.getName()), e);
             runtimeException = e;
         }
 
         // Release the filter chain (if any) for this request
-        try {
-            if (filterChain != null)
-                filterChain.release();
-        } catch (Throwable e) {
-            ExceptionUtils.handleThrowable(e);
-            wrapper.getLogger().error(sm.getString("standardWrapper.releaseFilters",
-                             wrapper.getName()), e);
-            // FIXME: Exception handling needs to be similar to what is in the StandardWrapperValue
+        if (filterChain != null) {
+            filterChain.release();
         }
 
         // Deallocate the allocated servlet instance
@@ -814,24 +759,25 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         }
 
         // Reset the old context class loader
-        if (oldCCL != null)
-            Thread.currentThread().setContextClassLoader(oldCCL);
-        
+        context.unbind(false, oldCCL);
+
         // Unwrap request/response if needed
         // See Bugzilla 30949
         unwrapRequest(state);
         unwrapResponse(state);
         // Recycle request if necessary (also BZ 30949)
         recycleRequestWrapper(state);
-        
-        // Rethrow an exception if one was thrown by the invoked servlet
-        if (ioException != null)
-            throw ioException;
-        if (servletException != null)
-            throw servletException;
-        if (runtimeException != null)
-            throw runtimeException;
 
+        // Rethrow an exception if one was thrown by the invoked servlet
+        if (ioException != null) {
+            throw ioException;
+        }
+        if (servletException != null) {
+            throw servletException;
+        }
+        if (runtimeException != null) {
+            throw runtimeException;
+        }
     }
 
 
@@ -840,8 +786,9 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
      */
     private void unwrapRequest(State state) {
 
-        if (state.wrapRequest == null)
+        if (state.wrapRequest == null) {
             return;
+        }
 
         if (state.outerRequest.isAsyncStarted()) {
             if (!state.outerRequest.getAsyncContext().hasOriginalRequestAndResponse()) {
@@ -854,18 +801,19 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         while (current != null) {
 
             // If we run into the container request we are done
-            if ((current instanceof Request)
-                || (current instanceof RequestFacade))
+            if ((current instanceof Request) || (current instanceof RequestFacade)) {
                 break;
+            }
 
             // Remove the current request if it is our wrapper
             if (current == state.wrapRequest) {
                 ServletRequest next =
                   ((ServletRequestWrapper) current).getRequest();
-                if (previous == null)
+                if (previous == null) {
                     state.outerRequest = next;
-                else
+                } else {
                     ((ServletRequestWrapper) previous).setRequest(next);
+                }
                 break;
             }
 
@@ -876,14 +824,15 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         }
 
     }
-    
+
     /**
      * Unwrap the response if we have wrapped it.
      */
     private void unwrapResponse(State state) {
 
-        if (state.wrapResponse == null)
+        if (state.wrapResponse == null) {
             return;
+        }
 
         if (state.outerRequest.isAsyncStarted()) {
             if (!state.outerRequest.getAsyncContext().hasOriginalRequestAndResponse()) {
@@ -896,18 +845,19 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         while (current != null) {
 
             // If we run into the container response we are done
-            if ((current instanceof Response)
-                || (current instanceof ResponseFacade))
+            if ((current instanceof Response) || (current instanceof ResponseFacade)) {
                 break;
+            }
 
             // Remove the current response if it is our wrapper
             if (current == state.wrapResponse) {
                 ServletResponse next =
                   ((ServletResponseWrapper) current).getResponse();
-                if (previous == null)
+                if (previous == null) {
                     state.outerResponse = next;
-                else
+                } else {
                     ((ServletResponseWrapper) previous).setResponse(next);
+                }
                 break;
             }
 
@@ -930,14 +880,18 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         ServletRequest previous = null;
         ServletRequest current = state.outerRequest;
         while (current != null) {
-            if(state.hrequest == null && (current instanceof HttpServletRequest))
+            if(state.hrequest == null && (current instanceof HttpServletRequest)) {
                 state.hrequest = (HttpServletRequest)current;
-            if (!(current instanceof ServletRequestWrapper))
+            }
+            if (!(current instanceof ServletRequestWrapper)) {
                 break;
-            if (current instanceof ApplicationHttpRequest)
+            }
+            if (current instanceof ApplicationHttpRequest) {
                 break;
-            if (current instanceof ApplicationRequest)
+            }
+            if (current instanceof ApplicationRequest) {
                 break;
+            }
             previous = current;
             current = ((ServletRequestWrapper) current).getRequest();
         }
@@ -953,7 +907,7 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
             if ((state.outerRequest instanceof ApplicationHttpRequest) ||
                 (state.outerRequest instanceof Request) ||
                 (state.outerRequest instanceof HttpServletRequest)) {
-                HttpServletRequest houterRequest = 
+                HttpServletRequest houterRequest =
                     (HttpServletRequest) state.outerRequest;
                 Object contextPath = houterRequest.getAttribute(
                         RequestDispatcher.INCLUDE_CONTEXT_PATH);
@@ -968,12 +922,13 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         } else {
             wrapper = new ApplicationRequest(current);
         }
-        if (previous == null)
+        if (previous == null) {
             state.outerRequest = wrapper;
-        else
+        } else {
             ((ServletRequestWrapper) previous).setRequest(wrapper);
+        }
         state.wrapRequest = wrapper;
-        return (wrapper);
+        return wrapper;
 
     }
 
@@ -988,37 +943,40 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
         ServletResponse previous = null;
         ServletResponse current = state.outerResponse;
         while (current != null) {
-            if(state.hresponse == null && (current instanceof HttpServletResponse)) {
+            if (state.hresponse == null && (current instanceof HttpServletResponse)) {
                 state.hresponse = (HttpServletResponse)current;
-                if(!state.including) // Forward only needs hresponse
+                if (!state.including) { // Forward only needs hresponse
                     return null;
+                }
             }
-            if (!(current instanceof ServletResponseWrapper))
+            if (!(current instanceof ServletResponseWrapper)) {
                 break;
-            if (current instanceof ApplicationHttpResponse)
+            }
+            if (current instanceof ApplicationHttpResponse) {
                 break;
-            if (current instanceof ApplicationResponse)
+            }
+            if (current instanceof ApplicationResponse) {
                 break;
+            }
             previous = current;
             current = ((ServletResponseWrapper) current).getResponse();
         }
 
         // Instantiate a new wrapper at this point and insert it in the chain
         ServletResponse wrapper = null;
-        if ((current instanceof ApplicationHttpResponse) ||
-            (current instanceof Response) ||
-            (current instanceof HttpServletResponse))
-            wrapper =
-                new ApplicationHttpResponse((HttpServletResponse) current,
-                        state.including);
-        else
+        if ((current instanceof ApplicationHttpResponse) || (current instanceof Response) ||
+                (current instanceof HttpServletResponse)) {
+            wrapper = new ApplicationHttpResponse((HttpServletResponse) current, state.including);
+        } else {
             wrapper = new ApplicationResponse(current, state.including);
-        if (previous == null)
+        }
+        if (previous == null) {
             state.outerResponse = wrapper;
-        else
+        } else {
             ((ServletResponseWrapper) previous).setResponse(wrapper);
+        }
         state.wrapResponse = wrapper;
-        return (wrapper);
+        return wrapper;
 
     }
 
@@ -1028,15 +986,15 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
             ApplicationFilterChain.getLastServicedRequest();
         ServletResponse originalResponse =
             ApplicationFilterChain.getLastServicedResponse();
-        
-        // Some forwards, eg from valves will not set original values 
+
+        // Some forwards, eg from valves will not set original values
         if (originalRequest == null || originalResponse == null) {
             return;
         }
-        
+
         boolean same = false;
         ServletRequest dispatchedRequest = appRequest;
-        
+
         //find the request that was passed into the service method
         while (originalRequest instanceof ServletRequestWrapper &&
                 ((ServletRequestWrapper) originalRequest).getRequest()!=null ) {
@@ -1059,13 +1017,13 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
             throw new ServletException(sm.getString(
                     "applicationDispatcher.specViolation.request"));
         }
-        
+
         same = false;
         ServletResponse dispatchedResponse = appResponse;
-        
+
         //find the response that was passed into the service method
         while (originalResponse instanceof ServletResponseWrapper &&
-                ((ServletResponseWrapper) originalResponse).getResponse() != 
+                ((ServletResponseWrapper) originalResponse).getResponse() !=
                     null ) {
             originalResponse =
                 ((ServletResponseWrapper) originalResponse).getResponse();
@@ -1075,7 +1033,7 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
             if (originalResponse.equals(dispatchedResponse)) {
                 same = true;
             }
-            
+
             if (!same && dispatchedResponse instanceof ServletResponseWrapper) {
                 dispatchedResponse =
                     ((ServletResponseWrapper) dispatchedResponse).getResponse();
@@ -1092,6 +1050,7 @@ final class ApplicationDispatcher implements AsyncDispatcher, RequestDispatcher
 
     private void recycleRequestWrapper(State state) {
         if (state.wrapRequest instanceof ApplicationHttpRequest) {
-            ((ApplicationHttpRequest) state.wrapRequest).recycle();        }
+            ((ApplicationHttpRequest) state.wrapRequest).recycle();
+        }
     }
 }
