@@ -16,7 +16,6 @@
  */
 package org.apache.catalina.core;
 
-import java.beans.Introspector;
 import java.io.IOException;
 import java.io.InputStream;
 import java.lang.annotation.Annotation;
@@ -28,11 +27,13 @@ import java.security.PrivilegedAction;
 import java.security.PrivilegedActionException;
 import java.security.PrivilegedExceptionAction;
 import java.util.ArrayList;
+import java.util.Collections;
 import java.util.HashMap;
+import java.util.HashSet;
 import java.util.List;
 import java.util.Map;
 import java.util.Properties;
-import java.util.WeakHashMap;
+import java.util.Set;
 
 import javax.annotation.PostConstruct;
 import javax.annotation.PreDestroy;
@@ -42,21 +43,18 @@ import javax.naming.Context;
 import javax.naming.NamingException;
 import javax.persistence.PersistenceContext;
 import javax.persistence.PersistenceUnit;
-import javax.servlet.Filter;
-import javax.servlet.Servlet;
 import javax.xml.ws.WebServiceRef;
 
 import org.apache.catalina.ContainerServlet;
 import org.apache.catalina.Globals;
 import org.apache.catalina.security.SecurityUtil;
 import org.apache.catalina.util.Introspection;
+import org.apache.juli.logging.Log;
 import org.apache.tomcat.InstanceManager;
 import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.collections.ManagedConcurrentWeakHashMap;
 import org.apache.tomcat.util.res.StringManager;
 
-/**
- * @version $Id: DefaultInstanceManager.java 1431300 2013-01-10 11:47:49Z markt $
- */
 public class DefaultInstanceManager implements InstanceManager {
 
     // Used when there are no annotations in a class
@@ -66,66 +64,71 @@ public class DefaultInstanceManager implements InstanceManager {
     /**
      * The string manager for this package.
      */
-    protected static final StringManager sm =
-        StringManager.getManager(Constants.Package);
+    protected static final StringManager sm = StringManager.getManager(DefaultInstanceManager.class);
+
+    private static final boolean EJB_PRESENT;
+    private static final boolean JPA_PRESENT;
+    private static final boolean WS_PRESENT;
+
+    static {
+        Class<?> clazz = null;
+        try {
+            clazz = Class.forName("javax.ejb.EJB");
+        } catch (ClassNotFoundException cnfe) {
+            // Expected
+        }
+        EJB_PRESENT = (clazz != null);
+
+        clazz = null;
+        try {
+            clazz = Class.forName("javax.persistence.PersistenceContext");
+        } catch (ClassNotFoundException cnfe) {
+            // Expected
+        }
+        JPA_PRESENT = (clazz != null);
+
+        clazz = null;
+        try {
+            clazz = Class.forName("javax.xml.ws.WebServiceRef");
+        } catch (ClassNotFoundException cnfe) {
+            // Expected
+        }
+        WS_PRESENT = (clazz != null);
+    }
+
 
     private final Context context;
     private final Map<String, Map<String, String>> injectionMap;
     protected final ClassLoader classLoader;
     protected final ClassLoader containerClassLoader;
-    protected boolean privileged;
-    protected boolean ignoreAnnotations;
-    private final Properties restrictedFilters = new Properties();
-    private final Properties restrictedListeners = new Properties();
-    private final Properties restrictedServlets = new Properties();
-    private final Map<Class<?>, AnnotationCacheEntry[]> annotationCache =
-        new WeakHashMap<Class<?>, AnnotationCacheEntry[]>();
+    protected final boolean privileged;
+    protected final boolean ignoreAnnotations;
+    private final Set<String> restrictedClasses;
+    private final ManagedConcurrentWeakHashMap<Class<?>, AnnotationCacheEntry[]> annotationCache =
+            new ManagedConcurrentWeakHashMap<>();
     private final Map<String, String> postConstructMethods;
     private final Map<String, String> preDestroyMethods;
 
-    public DefaultInstanceManager(Context context, Map<String, Map<String, String>> injectionMap, org.apache.catalina.Context catalinaContext, ClassLoader containerClassLoader) {
+    public DefaultInstanceManager(Context context,
+            Map<String, Map<String, String>> injectionMap,
+            org.apache.catalina.Context catalinaContext,
+            ClassLoader containerClassLoader) {
         classLoader = catalinaContext.getLoader().getClassLoader();
         privileged = catalinaContext.getPrivileged();
         this.containerClassLoader = containerClassLoader;
         ignoreAnnotations = catalinaContext.getIgnoreAnnotations();
-        StringManager sm = StringManager.getManager(Constants.Package);
-        try {
-            InputStream is =
-                this.getClass().getClassLoader().getResourceAsStream
-                    ("org/apache/catalina/core/RestrictedServlets.properties");
-            if (is != null) {
-                restrictedServlets.load(is);
-            } else {
-                catalinaContext.getLogger().error(sm.getString("defaultInstanceManager.restrictedServletsResource"));
-            }
-        } catch (IOException e) {
-            catalinaContext.getLogger().error(sm.getString("defaultInstanceManager.restrictedServletsResource"), e);
-        }
-
-        try {
-            InputStream is =
-                    this.getClass().getClassLoader().getResourceAsStream
-                            ("org/apache/catalina/core/RestrictedListeners.properties");
-            if (is != null) {
-                restrictedListeners.load(is);
-            } else {
-                catalinaContext.getLogger().error(sm.getString("defaultInstanceManager.restrictedListenersResources"));
-            }
-        } catch (IOException e) {
-            catalinaContext.getLogger().error(sm.getString("defaultInstanceManager.restrictedListenersResources"), e);
-        }
-        try {
-            InputStream is =
-                    this.getClass().getClassLoader().getResourceAsStream
-                            ("org/apache/catalina/core/RestrictedFilters.properties");
-            if (is != null) {
-                restrictedFilters.load(is);
-            } else {
-                catalinaContext.getLogger().error(sm.getString("defaultInstanceManager.restrictedFiltersResource"));
-            }
-        } catch (IOException e) {
-            catalinaContext.getLogger().error(sm.getString("defaultInstanceManager.restrictedServletsResources"), e);
-        }
+        Log log = catalinaContext.getLogger();
+        Set<String> classNames = new HashSet<>();
+        loadProperties(classNames,
+                "org/apache/catalina/core/RestrictedServlets.properties",
+                "defaultInstanceManager.restrictedServletsResource", log);
+        loadProperties(classNames,
+                "org/apache/catalina/core/RestrictedListeners.properties",
+                "defaultInstanceManager.restrictedListenersResource", log);
+        loadProperties(classNames,
+                "org/apache/catalina/core/RestrictedFilters.properties",
+                "defaultInstanceManager.restrictedFiltersResource", log);
+        restrictedClasses = Collections.unmodifiableSet(classNames);
         this.context = context;
         this.injectionMap = injectionMap;
         this.postConstructMethods = catalinaContext.findPostConstructMethods();
@@ -133,15 +136,27 @@ public class DefaultInstanceManager implements InstanceManager {
     }
 
     @Override
-    public Object newInstance(String className) throws IllegalAccessException, InvocationTargetException, NamingException, InstantiationException, ClassNotFoundException {
-        Class<?> clazz = loadClassMaybePrivileged(className, classLoader);
-        return newInstance(clazz.newInstance(), clazz);
+    public Object newInstance(Class<?> clazz) throws IllegalAccessException,
+            InvocationTargetException, NamingException, InstantiationException,
+            IllegalArgumentException, NoSuchMethodException, SecurityException {
+        return newInstance(clazz.getConstructor().newInstance(), clazz);
     }
 
     @Override
-    public Object newInstance(final String className, final ClassLoader classLoader) throws IllegalAccessException, NamingException, InvocationTargetException, InstantiationException, ClassNotFoundException {
+    public Object newInstance(String className) throws IllegalAccessException,
+            InvocationTargetException, NamingException, InstantiationException,
+            ClassNotFoundException, IllegalArgumentException, NoSuchMethodException, SecurityException {
+        Class<?> clazz = loadClassMaybePrivileged(className, classLoader);
+        return newInstance(clazz.getConstructor().newInstance(), clazz);
+    }
+
+    @Override
+    public Object newInstance(final String className, final ClassLoader classLoader)
+            throws IllegalAccessException, NamingException, InvocationTargetException,
+            InstantiationException, ClassNotFoundException, IllegalArgumentException,
+            NoSuchMethodException, SecurityException {
         Class<?> clazz = classLoader.loadClass(className);
-        return newInstance(clazz.newInstance(), clazz);
+        return newInstance(clazz.getConstructor().newInstance(), clazz);
     }
 
     @Override
@@ -150,7 +165,8 @@ public class DefaultInstanceManager implements InstanceManager {
         newInstance(o, o.getClass());
     }
 
-    private Object newInstance(Object instance, Class<?> clazz) throws IllegalAccessException, InvocationTargetException, NamingException {
+    private Object newInstance(Object instance, Class<?> clazz)
+            throws IllegalAccessException, InvocationTargetException, NamingException {
         if (!ignoreAnnotations) {
             Map<String, String> injections = assembleInjectionsFromClassHierarchy(clazz);
             populateAnnotationsCache(clazz, injections);
@@ -161,7 +177,7 @@ public class DefaultInstanceManager implements InstanceManager {
     }
 
     private Map<String, String> assembleInjectionsFromClassHierarchy(Class<?> clazz) {
-        Map<String, String> injections = new HashMap<String, String>();
+        Map<String, String> injections = new HashMap<>();
         Map<String, String> currentInjections = null;
         while (clazz != null) {
             currentInjections = this.injectionMap.get(clazz.getName());
@@ -174,14 +190,16 @@ public class DefaultInstanceManager implements InstanceManager {
     }
 
     @Override
-    public void destroyInstance(Object instance) throws IllegalAccessException, InvocationTargetException {
+    public void destroyInstance(Object instance) throws IllegalAccessException,
+            InvocationTargetException {
         if (!ignoreAnnotations) {
             preDestroy(instance, instance.getClass());
         }
     }
 
     /**
-     * Call postConstruct method on the specified instance recursively from deepest superclass to actual class.
+     * Call postConstruct method on the specified instance recursively from
+     * deepest superclass to actual class.
      *
      * @param instance object to call postconstruct methods on
      * @param clazz    (super) class to examine for postConstruct annotation.
@@ -203,10 +221,7 @@ public class DefaultInstanceManager implements InstanceManager {
 
         // At the end the postconstruct annotated
         // method is invoked
-        AnnotationCacheEntry[] annotations;
-        synchronized (annotationCache) {
-            annotations = annotationCache.get(clazz);
-        }
+        AnnotationCacheEntry[] annotations = annotationCache.get(clazz);
         for (AnnotationCacheEntry entry : annotations) {
             if (entry.getType() == AnnotationCacheEntryType.POST_CONSTRUCT) {
                 Method postConstruct = getMethod(clazz, entry);
@@ -222,7 +237,8 @@ public class DefaultInstanceManager implements InstanceManager {
 
 
     /**
-     * Call preDestroy method on the specified instance recursively from deepest superclass to actual class.
+     * Call preDestroy method on the specified instance recursively from deepest
+     * superclass to actual class.
      *
      * @param instance object to call preDestroy methods on
      * @param clazz    (super) class to examine for preDestroy annotation.
@@ -239,10 +255,7 @@ public class DefaultInstanceManager implements InstanceManager {
 
         // At the end the postconstruct annotated
         // method is invoked
-        AnnotationCacheEntry[] annotations = null;
-        synchronized (annotationCache) {
-            annotations = annotationCache.get(clazz);
-        }
+        AnnotationCacheEntry[] annotations = annotationCache.get(clazz);
         if (annotations == null) {
             // instance not created through the instance manager
             return;
@@ -261,6 +274,12 @@ public class DefaultInstanceManager implements InstanceManager {
     }
 
 
+    @Override
+    public void backgroundProcess() {
+        annotationCache.maintain();
+    }
+
+
     /**
      * Make sure that the annotations cache has been populated for the provided
      * class.
@@ -278,61 +297,17 @@ public class DefaultInstanceManager implements InstanceManager {
             InvocationTargetException, NamingException {
 
         List<AnnotationCacheEntry> annotations = null;
+        Set<String> injectionsMatchedToSetter = new HashSet<>();
 
         while (clazz != null) {
-            AnnotationCacheEntry[] annotationsArray = null;
-            synchronized (annotationCache) {
-                annotationsArray = annotationCache.get(clazz);
-            }
+            AnnotationCacheEntry[] annotationsArray = annotationCache.get(clazz);
             if (annotationsArray == null) {
                 if (annotations == null) {
-                    annotations = new ArrayList<AnnotationCacheEntry>();
+                    annotations = new ArrayList<>();
                 } else {
                     annotations.clear();
                 }
 
-                if (context != null) {
-                    // Initialize fields annotations for resource injection if
-                    // JNDI is enabled
-                    Field[] fields = Introspection.getDeclaredFields(clazz);
-                    for (Field field : fields) {
-                        if (injections != null && injections.containsKey(field.getName())) {
-                            annotations.add(new AnnotationCacheEntry(
-                                    field.getName(), null,
-                                    injections.get(field.getName()),
-                                    AnnotationCacheEntryType.FIELD));
-                        } else if (field.isAnnotationPresent(Resource.class)) {
-                            Resource annotation = field.getAnnotation(Resource.class);
-                            annotations.add(new AnnotationCacheEntry(
-                                    field.getName(), null, annotation.name(),
-                                    AnnotationCacheEntryType.FIELD));
-                        } else if (field.isAnnotationPresent(EJB.class)) {
-                            EJB annotation = field.getAnnotation(EJB.class);
-                            annotations.add(new AnnotationCacheEntry(
-                                    field.getName(), null, annotation.name(),
-                                    AnnotationCacheEntryType.FIELD));
-                        } else if (field.isAnnotationPresent(WebServiceRef.class)) {
-                            WebServiceRef annotation =
-                                    field.getAnnotation(WebServiceRef.class);
-                            annotations.add(new AnnotationCacheEntry(
-                                    field.getName(), null, annotation.name(),
-                                    AnnotationCacheEntryType.FIELD));
-                        } else if (field.isAnnotationPresent(PersistenceContext.class)) {
-                            PersistenceContext annotation =
-                                    field.getAnnotation(PersistenceContext.class);
-                            annotations.add(new AnnotationCacheEntry(
-                                    field.getName(), null, annotation.name(),
-                                    AnnotationCacheEntryType.FIELD));
-                        } else if (field.isAnnotationPresent(PersistenceUnit.class)) {
-                            PersistenceUnit annotation =
-                                    field.getAnnotation(PersistenceUnit.class);
-                            annotations.add(new AnnotationCacheEntry(
-                                    field.getName(), null, annotation.name(),
-                                    AnnotationCacheEntryType.FIELD));
-                        }
-                    }
-                }
-
                 // Initialize methods annotations
                 Method[] methods = Introspection.getDeclaredMethods(clazz);
                 Method postConstruct = null;
@@ -342,55 +317,56 @@ public class DefaultInstanceManager implements InstanceManager {
                 for (Method method : methods) {
                     if (context != null) {
                         // Resource injection only if JNDI is enabled
-                        if (injections != null &&
-                                Introspection.isValidSetter(method)) {
+                        if (injections != null && Introspection.isValidSetter(method)) {
                             String fieldName = Introspection.getPropertyName(method);
+                            injectionsMatchedToSetter.add(fieldName);
                             if (injections.containsKey(fieldName)) {
                                 annotations.add(new AnnotationCacheEntry(
                                         method.getName(),
                                         method.getParameterTypes(),
-                                        injections.get(method.getName()),
+                                        injections.get(fieldName),
                                         AnnotationCacheEntryType.SETTER));
-                                break;
+                                continue;
                             }
                         }
-                        if (method.isAnnotationPresent(Resource.class)) {
-                            Resource annotation = method.getAnnotation(Resource.class);
+                        Resource resourceAnnotation;
+                        Annotation ejbAnnotation;
+                        Annotation webServiceRefAnnotation;
+                        Annotation persistenceContextAnnotation;
+                        Annotation persistenceUnitAnnotation;
+                        if ((resourceAnnotation = method.getAnnotation(Resource.class)) != null) {
                             annotations.add(new AnnotationCacheEntry(
                                     method.getName(),
                                     method.getParameterTypes(),
-                                    annotation.name(),
+                                    resourceAnnotation.name(),
                                     AnnotationCacheEntryType.SETTER));
-                        } else if (method.isAnnotationPresent(EJB.class)) {
-                            EJB annotation = method.getAnnotation(EJB.class);
+                        } else if (EJB_PRESENT &&
+                                (ejbAnnotation = method.getAnnotation(EJB.class)) != null) {
                             annotations.add(new AnnotationCacheEntry(
                                     method.getName(),
                                     method.getParameterTypes(),
-                                    annotation.name(),
+                                    ((EJB) ejbAnnotation).name(),
                                     AnnotationCacheEntryType.SETTER));
-                        } else if (method.isAnnotationPresent(WebServiceRef.class)) {
-                            WebServiceRef annotation =
-                                    method.getAnnotation(WebServiceRef.class);
+                        } else if (WS_PRESENT && (webServiceRefAnnotation =
+                                method.getAnnotation(WebServiceRef.class)) != null) {
                             annotations.add(new AnnotationCacheEntry(
                                     method.getName(),
                                     method.getParameterTypes(),
-                                    annotation.name(),
+                                    ((WebServiceRef) webServiceRefAnnotation).name(),
                                     AnnotationCacheEntryType.SETTER));
-                        } else if (method.isAnnotationPresent(PersistenceContext.class)) {
-                            PersistenceContext annotation =
-                                    method.getAnnotation(PersistenceContext.class);
+                        } else if (JPA_PRESENT && (persistenceContextAnnotation =
+                                method.getAnnotation(PersistenceContext.class)) != null) {
                             annotations.add(new AnnotationCacheEntry(
                                     method.getName(),
                                     method.getParameterTypes(),
-                                    annotation.name(),
+                                    ((PersistenceContext) persistenceContextAnnotation).name(),
                                     AnnotationCacheEntryType.SETTER));
-                        } else if (method.isAnnotationPresent(PersistenceUnit.class)) {
-                            PersistenceUnit annotation =
-                                    method.getAnnotation(PersistenceUnit.class);
+                        } else if (JPA_PRESENT && (persistenceUnitAnnotation =
+                                method.getAnnotation(PersistenceUnit.class)) != null) {
                             annotations.add(new AnnotationCacheEntry(
                                     method.getName(),
                                     method.getParameterTypes(),
-                                    annotation.name(),
+                                    ((PersistenceUnit) persistenceUnitAnnotation).name(),
                                     AnnotationCacheEntryType.SETTER));
                         }
                     }
@@ -406,9 +382,8 @@ public class DefaultInstanceManager implements InstanceManager {
                             postConstruct.getParameterTypes(), null,
                             AnnotationCacheEntryType.POST_CONSTRUCT));
                 } else if (postConstructFromXml != null) {
-                    throw new IllegalArgumentException("Post construct method "
-                        + postConstructFromXml + " for class " + clazz.getName()
-                        + " is declared in deployment descriptor but cannot be found.");
+                    throw new IllegalArgumentException(sm.getString("defaultInstanceManager.postConstructNotFound",
+                        postConstructFromXml, clazz.getName()));
                 }
                 if (preDestroy != null) {
                     annotations.add(new AnnotationCacheEntry(
@@ -416,16 +391,58 @@ public class DefaultInstanceManager implements InstanceManager {
                             preDestroy.getParameterTypes(), null,
                             AnnotationCacheEntryType.PRE_DESTROY));
                 } else if (preDestroyFromXml != null) {
-                    throw new IllegalArgumentException("Pre destroy method "
-                        + preDestroyFromXml + " for class " + clazz.getName()
-                        + " is declared in deployment descriptor but cannot be found.");
+                    throw new IllegalArgumentException(sm.getString("defaultInstanceManager.preDestroyNotFound",
+                        preDestroyFromXml, clazz.getName()));
                 }
+
+                if (context != null) {
+                    // Initialize fields annotations for resource injection if
+                    // JNDI is enabled
+                    Field[] fields = Introspection.getDeclaredFields(clazz);
+                    for (Field field : fields) {
+                        Resource resourceAnnotation;
+                        Annotation ejbAnnotation;
+                        Annotation webServiceRefAnnotation;
+                        Annotation persistenceContextAnnotation;
+                        Annotation persistenceUnitAnnotation;
+                        String fieldName = field.getName();
+                        if (injections != null && injections.containsKey(fieldName) && !injectionsMatchedToSetter.contains(fieldName)) {
+                            annotations.add(new AnnotationCacheEntry(
+                                    fieldName, null,
+                                    injections.get(fieldName),
+                                    AnnotationCacheEntryType.FIELD));
+                        } else if ((resourceAnnotation =
+                                field.getAnnotation(Resource.class)) != null) {
+                            annotations.add(new AnnotationCacheEntry(fieldName, null,
+                                    resourceAnnotation.name(), AnnotationCacheEntryType.FIELD));
+                        } else if (EJB_PRESENT &&
+                                (ejbAnnotation = field.getAnnotation(EJB.class)) != null) {
+                            annotations.add(new AnnotationCacheEntry(fieldName, null,
+                                    ((EJB) ejbAnnotation).name(), AnnotationCacheEntryType.FIELD));
+                        } else if (WS_PRESENT && (webServiceRefAnnotation =
+                                field.getAnnotation(WebServiceRef.class)) != null) {
+                            annotations.add(new AnnotationCacheEntry(fieldName, null,
+                                    ((WebServiceRef) webServiceRefAnnotation).name(),
+                                    AnnotationCacheEntryType.FIELD));
+                        } else if (JPA_PRESENT && (persistenceContextAnnotation =
+                                field.getAnnotation(PersistenceContext.class)) != null) {
+                            annotations.add(new AnnotationCacheEntry(fieldName, null,
+                                    ((PersistenceContext) persistenceContextAnnotation).name(),
+                                    AnnotationCacheEntryType.FIELD));
+                        } else if (JPA_PRESENT && (persistenceUnitAnnotation =
+                                field.getAnnotation(PersistenceUnit.class)) != null) {
+                            annotations.add(new AnnotationCacheEntry(fieldName, null,
+                                    ((PersistenceUnit) persistenceUnitAnnotation).name(),
+                                    AnnotationCacheEntryType.FIELD));
+                        }
+                    }
+                }
+
                 if (annotations.isEmpty()) {
                     // Use common object to save memory
                     annotationsArray = ANNOTATIONS_EMPTY;
                 } else {
-                    annotationsArray = annotations.toArray(
-                            new AnnotationCacheEntry[annotations.size()]);
+                    annotationsArray = annotations.toArray(new AnnotationCacheEntry[0]);
                 }
                 synchronized (annotationCache) {
                     annotationCache.put(clazz, annotationsArray);
@@ -457,10 +474,7 @@ public class DefaultInstanceManager implements InstanceManager {
         Class<?> clazz = instance.getClass();
 
         while (clazz != null) {
-            AnnotationCacheEntry[] annotations;
-            synchronized (annotationCache) {
-                annotations = annotationCache.get(clazz);
-            }
+            AnnotationCacheEntry[] annotations = annotationCache.get(clazz);
             for (AnnotationCacheEntry entry : annotations) {
                 if (entry.getType() == AnnotationCacheEntryType.SETTER) {
                     lookupMethodResource(context, instance,
@@ -479,25 +493,21 @@ public class DefaultInstanceManager implements InstanceManager {
 
     /**
      * Makes cache size available to unit tests.
+     *
+     * @return the cache size
      */
     protected int getAnnotationCacheSize() {
-        synchronized (annotationCache) {
-            return annotationCache.size();
-        }
+        return annotationCache.size();
     }
 
 
-    protected Class<?> loadClassMaybePrivileged(final String className, final ClassLoader classLoader) throws ClassNotFoundException {
+    protected Class<?> loadClassMaybePrivileged(final String className,
+            final ClassLoader classLoader) throws ClassNotFoundException {
         Class<?> clazz;
         if (SecurityUtil.isPackageProtectionEnabled()) {
             try {
-                clazz = AccessController.doPrivileged(new PrivilegedExceptionAction<Class<?>>() {
-
-                    @Override
-                    public Class<?> run() throws Exception {
-                        return loadClass(className, classLoader);
-                    }
-                });
+                clazz = AccessController.doPrivileged(
+                        new PrivilegedLoadClass(className, classLoader));
             } catch (PrivilegedActionException e) {
                 Throwable t = e.getCause();
                 if (t instanceof ClassNotFoundException) {
@@ -512,7 +522,8 @@ public class DefaultInstanceManager implements InstanceManager {
         return clazz;
     }
 
-    protected Class<?> loadClass(String className, ClassLoader classLoader) throws ClassNotFoundException {
+    protected Class<?> loadClass(String className, ClassLoader classLoader)
+            throws ClassNotFoundException {
         if (className.startsWith("org.apache.catalina")) {
             return containerClassLoader.loadClass(className);
         }
@@ -531,27 +542,17 @@ public class DefaultInstanceManager implements InstanceManager {
         if (privileged) {
             return;
         }
-        if (Filter.class.isAssignableFrom(clazz)) {
-            checkAccess(clazz, restrictedFilters);
-        } else if (Servlet.class.isAssignableFrom(clazz)) {
-            if (ContainerServlet.class.isAssignableFrom(clazz)) {
-                throw new SecurityException("Restricted (ContainerServlet) " +
-                        clazz);
-            }
-            checkAccess(clazz, restrictedServlets);
-        } else {
-            checkAccess(clazz, restrictedListeners);
+        if (ContainerServlet.class.isAssignableFrom(clazz)) {
+            throw new SecurityException(sm.getString(
+                    "defaultInstanceManager.restrictedContainerServlet", clazz));
         }
-    }
-
-    private void checkAccess(Class<?> clazz, Properties restricted) {
         while (clazz != null) {
-            if ("restricted".equals(restricted.getProperty(clazz.getName()))) {
-                throw new SecurityException("Restricted " + clazz);
+            if (restrictedClasses.contains(clazz.getName())) {
+                throw new SecurityException(sm.getString(
+                        "defaultInstanceManager.restrictedClass", clazz));
             }
             clazz = clazz.getSuperclass();
         }
-
     }
 
     /**
@@ -631,11 +632,31 @@ public class DefaultInstanceManager implements InstanceManager {
         }
     }
 
-    @Deprecated
-    public static String getName(Method setter) {
-        // Note: method signature has already been checked for correctness.
-        // The method name always starts with "set".
-        return Introspector.decapitalize(setter.getName().substring(3));
+    private static void loadProperties(Set<String> classNames, String resourceName,
+            String messageKey, Log log) {
+        Properties properties = new Properties();
+        ClassLoader cl = DefaultInstanceManager.class.getClassLoader();
+        try (InputStream is = cl.getResourceAsStream(resourceName)) {
+            if (is == null) {
+                log.error(sm.getString(messageKey, resourceName));
+            } else {
+                properties.load(is);
+            }
+        } catch (IOException ioe) {
+            log.error(sm.getString(messageKey, resourceName), ioe);
+        }
+        if (properties.isEmpty()) {
+            return;
+        }
+        for (Map.Entry<Object, Object> e : properties.entrySet()) {
+            if ("restricted".equals(e.getValue())) {
+                classNames.add(e.getKey().toString());
+            } else {
+                log.warn(sm.getString(
+                        "defaultInstanceManager.restrictedWrongValue",
+                        resourceName, e.getKey(), e.getValue()));
+            }
+        }
     }
 
     private static String normalize(String jndiName){
@@ -649,22 +670,7 @@ public class DefaultInstanceManager implements InstanceManager {
             final AnnotationCacheEntry entry) {
         Method result = null;
         if (Globals.IS_SECURITY_ENABLED) {
-            result = AccessController.doPrivileged(
-                    new PrivilegedAction<Method>() {
-                        @Override
-                        public Method run() {
-                            Method result = null;
-                            try {
-                                result = clazz.getDeclaredMethod(
-                                        entry.getAccessibleObjectName(),
-                                        entry.getParamTypes());
-                            } catch (NoSuchMethodException e) {
-                                // Should never happen. On that basis don't log
-                                // it.
-                            }
-                            return result;
-                        }
-            });
+            result = AccessController.doPrivileged(new PrivilegedGetMethod(clazz, entry));
         } else {
             try {
                 result = clazz.getDeclaredMethod(
@@ -680,25 +686,10 @@ public class DefaultInstanceManager implements InstanceManager {
             final AnnotationCacheEntry entry) {
         Field result = null;
         if (Globals.IS_SECURITY_ENABLED) {
-            result = AccessController.doPrivileged(
-                    new PrivilegedAction<Field>() {
-                        @Override
-                        public Field run() {
-                            Field result = null;
-                            try {
-                                result = clazz.getDeclaredField(
-                                        entry.getAccessibleObjectName());
-                            } catch (NoSuchFieldException e) {
-                                // Should never happen. On that basis don't log
-                                // it.
-                            }
-                            return result;
-                        }
-            });
+            result = AccessController.doPrivileged(new PrivilegedGetField(clazz, entry));
         } else {
             try {
-                result = clazz.getDeclaredField(
-                        entry.getAccessibleObjectName());
+                result = clazz.getDeclaredField(entry.getAccessibleObjectName());
             } catch (NoSuchFieldException e) {
                 // Should never happen. On that basis don't log it.
             }
@@ -727,16 +718,15 @@ public class DefaultInstanceManager implements InstanceManager {
             if (method.getName().equals(methodNameFromXml)) {
                 if (!Introspection.isValidLifecycleCallback(method)) {
                     throw new IllegalArgumentException(
-                        "Invalid " + annotation.getName() + " annotation");
+                            "Invalid " + annotation.getName() + " annotation");
                 }
                 result = method;
             }
         } else {
             if (method.isAnnotationPresent(annotation)) {
-                if (currentMethod != null ||
-                    !Introspection.isValidLifecycleCallback(method)) {
+                if (currentMethod != null || !Introspection.isValidLifecycleCallback(method)) {
                     throw new IllegalArgumentException(
-                        "Invalid " + annotation.getName() + " annotation");
+                            "Invalid " + annotation.getName() + " annotation");
                 }
                 result = method;
             }
@@ -775,7 +765,72 @@ public class DefaultInstanceManager implements InstanceManager {
         }
     }
 
-    private static enum AnnotationCacheEntryType {
+
+    private enum AnnotationCacheEntryType {
         FIELD, SETTER, POST_CONSTRUCT, PRE_DESTROY
     }
+
+
+    private static class PrivilegedGetField implements PrivilegedAction<Field> {
+
+        private final Class<?> clazz;
+        private final AnnotationCacheEntry entry;
+
+        public PrivilegedGetField(Class<?> clazz, AnnotationCacheEntry entry) {
+            this.clazz = clazz;
+            this.entry = entry;
+        }
+
+        @Override
+        public Field run() {
+            Field result = null;
+            try {
+                result = clazz.getDeclaredField(entry.getAccessibleObjectName());
+            } catch (NoSuchFieldException e) {
+                // Should never happen. On that basis don't log it.
+            }
+            return result;
+        }
+    }
+
+
+    private static class PrivilegedGetMethod implements PrivilegedAction<Method> {
+
+        private final Class<?> clazz;
+        private final AnnotationCacheEntry entry;
+
+        public PrivilegedGetMethod(Class<?> clazz, AnnotationCacheEntry entry) {
+            this.clazz = clazz;
+            this.entry = entry;
+        }
+
+        @Override
+        public Method run() {
+            Method result = null;
+            try {
+                result = clazz.getDeclaredMethod(
+                        entry.getAccessibleObjectName(), entry.getParamTypes());
+            } catch (NoSuchMethodException e) {
+                // Should never happen. On that basis don't log it.
+            }
+            return result;
+        }
+    }
+
+
+    private class PrivilegedLoadClass implements PrivilegedExceptionAction<Class<?>> {
+
+        private final String className;
+        private final ClassLoader classLoader;
+
+        public PrivilegedLoadClass(String className, ClassLoader classLoader) {
+            this.className = className;
+            this.classLoader = classLoader;
+        }
+
+        @Override
+        public Class<?> run() throws Exception {
+            return loadClass(className, classLoader);
+        }
+    }
 }
