@@ -14,30 +14,34 @@
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
 package org.apache.catalina.core;
 
 import java.io.IOException;
 import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
-import java.net.MalformedURLException;
-import java.net.URL;
 import java.net.URLConnection;
 import java.sql.DriverManager;
 import java.util.StringTokenizer;
+import java.util.concurrent.ForkJoinPool;
 
 import javax.imageio.ImageIO;
+import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import javax.xml.parsers.ParserConfigurationException;
 
-import org.apache.catalina.Globals;
 import org.apache.catalina.Lifecycle;
 import org.apache.catalina.LifecycleEvent;
 import org.apache.catalina.LifecycleListener;
+import org.apache.catalina.Server;
+import org.apache.catalina.startup.SafeForkJoinWorkerThreadFactory;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.compat.JreCompat;
+import org.apache.tomcat.util.compat.JreVendor;
 import org.apache.tomcat.util.res.StringManager;
+import org.w3c.dom.Document;
+import org.w3c.dom.ls.DOMImplementationLS;
 
 /**
  * Provide a workaround for known places where the Java Runtime environment can
@@ -51,39 +55,24 @@ import org.apache.tomcat.util.res.StringManager;
  * <p>
  * Locked files usually occur when a resource inside a JAR is accessed without
  * first disabling Jar URL connection caching. The workaround is to disable this
- * caching by default. 
+ * caching by default.
+ * <p>
+ * This listener must only be nested within {@link Server} elements.
  */
 public class JreMemoryLeakPreventionListener implements LifecycleListener {
 
-    private static final Log log =
-        LogFactory.getLog(JreMemoryLeakPreventionListener.class);
-    private static final StringManager sm =
-        StringManager.getManager(Constants.Package);
-
-    private static final boolean IS_JAVA_7_OR_LATER;
-
-    static {
-        boolean isJava7OrLater;
-        try {
-            Class.forName("java.util.Objects");
-            isJava7OrLater = true;
-        } catch (ClassNotFoundException e) {
-            isJava7OrLater = false;
-        }
-        IS_JAVA_7_OR_LATER = isJava7OrLater;
-    }
+    private static final Log log = LogFactory.getLog(JreMemoryLeakPreventionListener.class);
+    private static final StringManager sm = StringManager.getManager(JreMemoryLeakPreventionListener.class);
 
+    private static final String FORK_JOIN_POOL_THREAD_FACTORY_PROPERTY =
+            "java.util.concurrent.ForkJoinPool.common.threadFactory";
     /**
      * Protect against the memory leak caused when the first call to
      * <code>sun.awt.AppContext.getAppContext()</code> is triggered by a web
-     * application. Defaults to <code>true</code> for Java 6 and earlier (since
-     * it is used by {@link java.beans.Introspector#flushCaches()}) but defaults
-     * to <code>false</code> for Java 7 and later since
-     * {@link java.beans.Introspector#flushCaches()} no longer uses AppContext
-     * from 1.7.0_02 onwards. Also, from 1.7.0_25 onwards, calling this method
-     * requires a graphical environment and starts an AWT thread.
+     * application. Defaults to <code>false</code> since Tomcat code no longer
+     * triggers this although application code may.
      */
-    private boolean appContextProtection = !IS_JAVA_7_OR_LATER;
+    private boolean appContextProtection = false;
     public boolean isAppContextProtection() { return appContextProtection; }
     public void setAppContextProtection(boolean appContextProtection) {
         this.appContextProtection = appContextProtection;
@@ -101,25 +90,14 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
       this.awtThreadProtection = awtThreadProtection;
     }
 
-    /**
-     * Protect against the memory leak caused when the
-     * <code>sun.java2d.Disposer</code> class is loaded by a web application.
-     * Defaults to <code>false</code> because a new Thread is launched.
-     */
-    private boolean java2dDisposerProtection = false;
-    public boolean isJava2DDisposerProtection() {
-        return java2dDisposerProtection;
-    }
-    public void setJava2DDisposerProtection(boolean java2dDisposerProtection) {
-        this.java2dDisposerProtection = java2dDisposerProtection;
-    }
-
     /**
      * Protect against the memory leak caused when the first call to
      * <code>sun.misc.GC.requestLatency(long)</code> is triggered by a web
      * application. This first call will start a GC Daemon thread with the
      * thread's context class loader configured to be the web application class
      * loader. Defaults to <code>true</code>.
+     *
+     * @see "http://bugs.java.com/bugdatabase/view_bug.do?bug_id=JDK-8157570"
      */
     private boolean gcDaemonProtection = true;
     public boolean isGcDaemonProtection() { return gcDaemonProtection; }
@@ -127,35 +105,6 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
         this.gcDaemonProtection = gcDaemonProtection;
     }
 
-     /**
-      * Protect against the memory leak caused when the first call to
-      * <code>javax.security.auth.Policy</code> is triggered by a web
-      * application. This first call populate a static variable with a reference
-      * to the context class loader. Defaults to <code>true</code>.
-      */
-     private boolean securityPolicyProtection = true;
-     public boolean isSecurityPolicyProtection() {
-         return securityPolicyProtection;
-     }
-     public void setSecurityPolicyProtection(boolean securityPolicyProtection) {
-         this.securityPolicyProtection = securityPolicyProtection;
-     }
-     
-    /**
-     * Protects against the memory leak caused when the first call to
-     * <code>javax.security.auth.login.Configuration</code> is triggered by a
-     * web application. This first call populate a static variable with a
-     * reference to the context class loader. Defaults to <code>true</code>.
-     */
-    private boolean securityLoginConfigurationProtection = true;
-    public boolean isSecurityLoginConfigurationProtection() {
-        return securityLoginConfigurationProtection;
-    }
-    public void setSecurityLoginConfigurationProtection(
-            boolean securityLoginConfigurationProtection) {
-        this.securityLoginConfigurationProtection = securityLoginConfigurationProtection;
-    }
-
      /**
      * Protect against the memory leak, when the initialization of the
      * Java Cryptography Architecture is triggered by initializing
@@ -184,25 +133,27 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
     }
 
     /**
-     * XML parsing can pin a web application class loader in memory. This is
-     * particularly nasty as profilers (at least YourKit and Eclipse MAT) don't
-     * identify any GC roots related to this.
-     * <a href="http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6916498">
-     * http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=6916498</a>
+     * XML parsing can pin a web application class loader in memory. There are
+     * multiple root causes for this. Some of these are particularly nasty as
+     * profilers may not identify any GC roots related to the leak. For example,
+     * with YourKit you need to ensure that HPROF format memory snapshots are
+     * used to be able to trace some of the leaks.
      */
     private boolean xmlParsingProtection = true;
     public boolean isXmlParsingProtection() { return xmlParsingProtection; }
     public void setXmlParsingProtection(boolean xmlParsingProtection) {
         this.xmlParsingProtection = xmlParsingProtection;
     }
-    
+
     /**
      * <code>com.sun.jndi.ldap.LdapPoolManager</code> class spawns a thread when
      * it is initialized if the system property
      * <code>com.sun.jndi.ldap.connect.pool.timeout</code> is greater than 0.
      * That thread inherits the context class loader of the current thread, so
      * that there may be a web application class loader leak if the web app
-     * is the first to use <code>LdapPoolManager</code>. 
+     * is the first to use <code>LdapPoolManager</code>.
+     *
+     * @see "http://bugs.java.com/bugdatabase/view_bug.do?bug_id=JDK-8156824"
      */
     private boolean ldapPoolProtection = true;
     public boolean isLdapPoolProtection() { return ldapPoolProtection; }
@@ -214,7 +165,7 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
      * The first access to {@link DriverManager} will trigger the loading of
      * all {@link java.sql.Driver}s in the the current class loader. The web
      * application level memory leak protection can take care of this in most
-     * cases but triggering the loading here has fewer side-effects. 
+     * cases but triggering the loading here has fewer side-effects.
      */
     private boolean driverManagerProtection = true;
     public boolean isDriverManagerProtection() {
@@ -223,10 +174,25 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
     public void setDriverManagerProtection(boolean driverManagerProtection) {
         this.driverManagerProtection = driverManagerProtection;
     }
-    
+
+    /**
+     * {@link ForkJoinPool#commonPool()} creates a thread pool that, by default,
+     * creates threads that retain references to the thread context class
+     * loader.
+     *
+     * @see "http://bugs.java.com/bugdatabase/view_bug.do?bug_id=JDK-8172726"
+     */
+    private boolean forkJoinCommonPoolProtection = true;
+    public boolean getForkJoinCommonPoolProtection() {
+        return forkJoinCommonPoolProtection;
+    }
+    public void setForkJoinCommonPoolProtection(boolean forkJoinCommonPoolProtection) {
+        this.forkJoinCommonPoolProtection = forkJoinCommonPoolProtection;
+    }
+
     /**
      * List of comma-separated fully qualified class names to load and initialize during
-     * the startup of this Listener. This allows to pre-load classes that are known to 
+     * the startup of this Listener. This allows to pre-load classes that are known to
      * provoke classloader leaks if they are loaded during a request processing.
      */
     private String classesToInitialize = null;
@@ -236,11 +202,32 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
     public void setClassesToInitialize(String classesToInitialize) {
         this.classesToInitialize = classesToInitialize;
     }
-    
+
+
+
     @Override
     public void lifecycleEvent(LifecycleEvent event) {
         // Initialise these classes when Tomcat starts
         if (Lifecycle.BEFORE_INIT_EVENT.equals(event.getType())) {
+            if (!(event.getLifecycle() instanceof Server)) {
+                log.warn(sm.getString("listener.notServer",
+                        event.getLifecycle().getClass().getSimpleName()));
+            }
+
+            /*
+             * First call to this loads all drivers visible to the current class
+             * loader and its parents.
+             *
+             * Note: This is called before the context class loader is changed
+             *       because we want any drivers located in CATALINA_HOME/lib
+             *       and/or CATALINA_HOME/lib to be visible to DriverManager.
+             *       Users wishing to avoid having JDBC drivers loaded by this
+             *       class loader should add the JDBC driver(s) to the class
+             *       path so they are loaded by the system class loader.
+             */
+            if (driverManagerProtection) {
+                DriverManager.getDrivers();
+            }
 
             ClassLoader loader = Thread.currentThread().getContextClassLoader();
 
@@ -251,33 +238,21 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                 Thread.currentThread().setContextClassLoader(
                         ClassLoader.getSystemClassLoader());
 
-                /*
-                 * First call to this loads all drivers in the current class
-                 * loader
-                 */
-                if (driverManagerProtection) {
-                    DriverManager.getDrivers();
-                }
-
                 /*
                  * Several components end up calling:
                  * sun.awt.AppContext.getAppContext()
-                 * 
+                 *
                  * Those libraries / components known to trigger memory leaks
                  * due to eventual calls to getAppContext() are:
                  * - Google Web Toolkit via its use of javax.imageio
-                 * - Tomcat via its use of java.beans.Introspector.flushCaches()
-                 *   in 1.6.0_15 to 1.7.0_01. From 1.7.0_02 onwards use of
-                 *   AppContext by Introspector.flushCaches() was replaced with
-                 *   ThreadGroupContext
+                 * - Batik
                  * - others TBD
                  *
-                 * From 1.7.0_25 onwards, a call to
-                 * sun.awt.AppContext.getAppContext() results in a thread being
-                 * started named AWT-AppKit that requires a graphic environment
-                 * to be available.
+                 * Note tha a call to sun.awt.AppContext.getAppContext() results
+                 * in a thread being started named AWT-AppKit that requires a
+                 * graphical environment to be available.
                  */
-                
+
                 // Trigger a call to sun.awt.AppContext.getAppContext(). This
                 // will pin the system class loader in memory but that shouldn't
                 // be an issue.
@@ -286,28 +261,17 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                 }
 
                 // Trigger the creation of the AWT (AWT-Windows, AWT-XAWT,
-                // etc.) thread
-                if (awtThreadProtection) {
-                  java.awt.Toolkit.getDefaultToolkit();
-                }
-
-                // Trigger the creation of the "Java2D Disposer" thread.
-                // See https://issues.apache.org/bugzilla/show_bug.cgi?id=51687
-                if(java2dDisposerProtection) {
-                    try {
-                        Class.forName("sun.java2d.Disposer");
-                    }
-                    catch (ClassNotFoundException cnfe) {
-                        // Ignore this case: we must be running on a
-                        // non-Sun-based JRE.
-                    }
+                // etc.) thread.
+                // Note this issue is fixed in Java 8 update 05 onwards.
+                if (awtThreadProtection && !JreCompat.isJre9Available()) {
+                    java.awt.Toolkit.getDefaultToolkit();
                 }
 
                 /*
                  * Several components end up calling
                  * sun.misc.GC.requestLatency(long) which creates a daemon
                  * thread without setting the TCCL.
-                 * 
+                 *
                  * Those libraries / components known to trigger memory leaks
                  * due to eventual calls to requestLatency(long) are:
                  * - javax.management.remote.rmi.RMIConnectorServer.start()
@@ -315,8 +279,9 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                  * Note: Long.MAX_VALUE is a special case that causes the thread
                  *       to terminate
                  *
+                 * Fixed in Java 9 onwards (from early access build 130)
                  */
-                if (gcDaemonProtection) {
+                if (gcDaemonProtection && !JreCompat.isJre9Available()) {
                     try {
                         Class<?> clazz = Class.forName("sun.misc.GC");
                         Method method = clazz.getDeclaredMethod(
@@ -324,23 +289,15 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                                 new Class[] {long.class});
                         method.invoke(null, Long.valueOf(Long.MAX_VALUE - 1));
                     } catch (ClassNotFoundException e) {
-                        if (Globals.IS_ORACLE_JVM) {
+                        if (JreVendor.IS_ORACLE_JVM) {
                             log.error(sm.getString(
                                     "jreLeakListener.gcDaemonFail"), e);
                         } else {
                             log.debug(sm.getString(
                                     "jreLeakListener.gcDaemonFail"), e);
                         }
-                    } catch (SecurityException e) {
-                        log.error(sm.getString("jreLeakListener.gcDaemonFail"),
-                                e);
-                    } catch (NoSuchMethodException e) {
-                        log.error(sm.getString("jreLeakListener.gcDaemonFail"),
-                                e);
-                    } catch (IllegalArgumentException e) {
-                        log.error(sm.getString("jreLeakListener.gcDaemonFail"),
-                                e);
-                    } catch (IllegalAccessException e) {
+                    } catch (SecurityException | NoSuchMethodException | IllegalArgumentException |
+                            IllegalAccessException e) {
                         log.error(sm.getString("jreLeakListener.gcDaemonFail"),
                                 e);
                     } catch (InvocationTargetException e) {
@@ -349,114 +306,85 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                                 e);
                     }
                 }
-    
-                /*
-                 * Calling getPolicy retains a static reference to the context 
-                 * class loader.
-                 */
-                if (securityPolicyProtection) {
-                    try {
-                        // Policy.getPolicy();
-                        Class<?> policyClass = Class
-                                .forName("javax.security.auth.Policy");
-                        Method method = policyClass.getMethod("getPolicy");
-                        method.invoke(null);
-                    } catch(ClassNotFoundException e) {
-                        // Ignore. The class is deprecated.
-                    } catch(SecurityException e) {
-                        // Ignore. Don't need call to getPolicy() to be
-                        // successful, just need to trigger static initializer.
-                    } catch (NoSuchMethodException e) {
-                        log.warn(sm.getString("jreLeakListener.authPolicyFail"),
-                                e);
-                    } catch (IllegalArgumentException e) {
-                        log.warn(sm.getString("jreLeakListener.authPolicyFail"),
-                                e);
-                    } catch (IllegalAccessException e) {
-                        log.warn(sm.getString("jreLeakListener.authPolicyFail"),
-                                e);
-                    } catch (InvocationTargetException e) {
-                        ExceptionUtils.handleThrowable(e.getCause());
-                        log.warn(sm.getString("jreLeakListener.authPolicyFail"),
-                                e);
-                    }
-                }
-    
-                
-                /*
-                 * Initializing javax.security.auth.login.Configuration retains a static reference to the context 
-                 * class loader.
-                 */
-                if (securityLoginConfigurationProtection) {
-                    try {
-                        Class.forName("javax.security.auth.login.Configuration", true, ClassLoader.getSystemClassLoader());
-                    } catch(ClassNotFoundException e) {
-                        // Ignore
-                    }
-                }
 
                 /*
                  * Creating a MessageDigest during web application startup
                  * initializes the Java Cryptography Architecture. Under certain
                  * conditions this starts a Token poller thread with TCCL equal
                  * to the web application class loader.
-                 * 
+                 *
                  * Instead we initialize JCA right now.
+                 *
+                 * Fixed in Java 9 onwards (from early access build 133)
                  */
-                if (tokenPollerProtection) {
+                if (tokenPollerProtection && !JreCompat.isJre9Available()) {
                     java.security.Security.getProviders();
                 }
-                
+
                 /*
                  * Several components end up opening JarURLConnections without
                  * first disabling caching. This effectively locks the file.
                  * Whilst more noticeable and harder to ignore on Windows, it
                  * affects all operating systems.
-                 * 
+                 *
                  * Those libraries/components known to trigger this issue
                  * include:
                  * - log4j versions 1.2.15 and earlier
                  * - javax.xml.bind.JAXBContext.newInstance()
+                 *
+                 * https://bugs.openjdk.java.net/browse/JDK-8163449
+                 *
+                 * Java 9 onwards disables caching for JAR URLConnections
+                 * Java 8 and earlier disables caching for all URLConnections
                  */
-                
+
                 // Set the default URL caching policy to not to cache
                 if (urlCacheProtection) {
                     try {
-                        // Doesn't matter that this JAR doesn't exist - just as
-                        // long as the URL is well-formed
-                        URL url = new URL("jar:file://dummy.jar!/");
-                        URLConnection uConn = url.openConnection();
-                        uConn.setDefaultUseCaches(false);
-                    } catch (MalformedURLException e) {
-                        log.error(sm.getString(
-                                "jreLeakListener.jarUrlConnCacheFail"), e);
+                        JreCompat.getInstance().disableCachingForJarUrlConnections();
                     } catch (IOException e) {
-                        log.error(sm.getString(
-                                "jreLeakListener.jarUrlConnCacheFail"), e);
+                        log.error(sm.getString("jreLeakListener.jarUrlConnCacheFail"), e);
                     }
                 }
-                
+
                 /*
-                 * Haven't got to the root of what is going on with this leak
-                 * but if a web app is the first to make the calls below the web
-                 * application class loader will be pinned in memory.
+                 * Fixed in Java 9 onwards (from early access build 133)
                  */
-                if (xmlParsingProtection) {
-                    DocumentBuilderFactory factory =
-                        DocumentBuilderFactory.newInstance();
+                if (xmlParsingProtection && !JreCompat.isJre9Available()) {
+                    // There are two known issues with XML parsing that affect
+                    // Java 8+. The issues both relate to cached Exception
+                    // instances that retain a link to the TCCL via the
+                    // backtrace field. Note that YourKit only shows this field
+                    // when using the HPROF format memory snapshots.
+                    // https://bz.apache.org/bugzilla/show_bug.cgi?id=58486
+                    // https://bugs.openjdk.java.net/browse/JDK-8146961
+                    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
                     try {
-                        factory.newDocumentBuilder();
+                        DocumentBuilder documentBuilder = factory.newDocumentBuilder();
+                        // Issue 1
+                        // com.sun.org.apache.xml.internal.serialize.DOMSerializerImpl
+                        Document document = documentBuilder.newDocument();
+                        document.createElement("dummy");
+                        DOMImplementationLS implementation =
+                                (DOMImplementationLS)document.getImplementation();
+                        implementation.createLSSerializer().writeToString(document);
+                        // Issue 1
+                        // com.sun.org.apache.xerces.internal.dom.DOMNormalizer
+                        document.normalize();
                     } catch (ParserConfigurationException e) {
                         log.error(sm.getString("jreLeakListener.xmlParseFail"),
                                 e);
                     }
                 }
-                
-                if (ldapPoolProtection) {
+
+                /*
+                 * Fixed in Java 9 onwards (from early access build 130)
+                 */
+                if (ldapPoolProtection && !JreCompat.isJre9Available()) {
                     try {
                         Class.forName("com.sun.jndi.ldap.LdapPoolManager");
                     } catch (ClassNotFoundException e) {
-                        if (Globals.IS_ORACLE_JVM) {
+                        if (JreVendor.IS_ORACLE_JVM) {
                             log.error(sm.getString(
                                     "jreLeakListener.ldapPoolManagerFail"), e);
                         } else {
@@ -465,7 +393,19 @@ public class JreMemoryLeakPreventionListener implements LifecycleListener {
                         }
                     }
                 }
-                
+
+                /*
+                 * Present in Java 7 onwards
+                 * Fixed in Java 9 (from early access build 156)
+                 */
+                if (forkJoinCommonPoolProtection && !JreCompat.isJre9Available()) {
+                    // Don't override any explicitly set property
+                    if (System.getProperty(FORK_JOIN_POOL_THREAD_FACTORY_PROPERTY) == null) {
+                        System.setProperty(FORK_JOIN_POOL_THREAD_FACTORY_PROPERTY,
+                                SafeForkJoinWorkerThreadFactory.class.getName());
+                    }
+                }
+
                 if (classesToInitialize != null) {
                     StringTokenizer strTok =
                         new StringTokenizer(classesToInitialize, ", \r\n\t");
