@@ -5,21 +5,17 @@
  * The ASF licenses this file to You under the Apache License, Version 2.0
  * (the "License"); you may not use this file except in compliance with
  * the License.  You may obtain a copy of the License at
- * 
+ *
  *      http://www.apache.org/licenses/LICENSE-2.0
- * 
+ *
  * Unless required by applicable law or agreed to in writing, software
  * distributed under the License is distributed on an "AS IS" BASIS,
  * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  * See the License for the specific language governing permissions and
  * limitations under the License.
  */
-
-
 package org.apache.catalina.session;
 
-
-import java.io.BufferedInputStream;
 import java.io.BufferedOutputStream;
 import java.io.File;
 import java.io.FileInputStream;
@@ -29,15 +25,16 @@ import java.io.IOException;
 import java.io.ObjectInputStream;
 import java.io.ObjectOutputStream;
 import java.util.ArrayList;
+import java.util.List;
 
 import javax.servlet.ServletContext;
 
-import org.apache.catalina.Container;
 import org.apache.catalina.Context;
-import org.apache.catalina.Loader;
+import org.apache.catalina.Globals;
 import org.apache.catalina.Session;
-import org.apache.catalina.util.CustomObjectInputStream;
-
+import org.apache.juli.logging.Log;
+import org.apache.juli.logging.LogFactory;
+import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Concrete implementation of the <b>Store</b> interface that utilizes
@@ -48,10 +45,12 @@ import org.apache.catalina.util.CustomObjectInputStream;
  */
 public final class FileStore extends StoreBase {
 
+    private static final Log log = LogFactory.getLog(FileStore.class);
+    private static final StringManager sm = StringManager.getManager(FileStore.class);
+
 
     // ----------------------------------------------------- Constants
 
-
     /**
      * The extension to use for serialized session filenames.
      */
@@ -60,7 +59,6 @@ public final class FileStore extends StoreBase {
 
     // ----------------------------------------------------- Instance Variables
 
-
     /**
      * The pathname of the directory in which Sessions are stored.
      * This may be an absolute pathname, or a relative path that is
@@ -75,16 +73,12 @@ public final class FileStore extends StoreBase {
     private File directoryFile = null;
 
 
-    /**
-     * The descriptive information about this implementation.
-     */
-    private static final String info = "FileStore/1.0";
-
     /**
      * Name to register for this Store, used for logging.
      */
     private static final String storeName = "fileStore";
 
+
     /**
      * Name to register for the background thread.
      */
@@ -93,14 +87,11 @@ public final class FileStore extends StoreBase {
 
     // ------------------------------------------------------------- Properties
 
-
     /**
-     * Return the directory path for this Store.
+     * @return The directory path for this Store.
      */
     public String getDirectory() {
-
-        return (directory);
-
+        return directory;
     }
 
 
@@ -110,41 +101,27 @@ public final class FileStore extends StoreBase {
      * @param path The new directory path
      */
     public void setDirectory(String path) {
-
         String oldDirectory = this.directory;
         this.directory = path;
         this.directoryFile = null;
-        support.firePropertyChange("directory", oldDirectory,
-                                   this.directory);
-
+        support.firePropertyChange("directory", oldDirectory, this.directory);
     }
 
 
     /**
-     * Return descriptive information about this Store implementation and
-     * the corresponding version number, in the format
-     * <code>&lt;description&gt;/&lt;version&gt;</code>.
-     */
-    @Override
-    public String getInfo() {
-
-        return (info);
-
-    }
-
-    /**
-     * Return the thread name for this Store.
+     * @return The thread name for this Store.
      */
     public String getThreadName() {
-        return(threadName);
+        return threadName;
     }
 
+
     /**
      * Return the name for this Store, used for logging.
      */
     @Override
     public String getStoreName() {
-        return(storeName);
+        return storeName;
     }
 
 
@@ -155,43 +132,39 @@ public final class FileStore extends StoreBase {
      */
     @Override
     public int getSize() throws IOException {
-
         // Acquire the list of files in our storage directory
-        File file = directory();
-        if (file == null) {
-            return (0);
+        File dir = directory();
+        if (dir == null) {
+            return 0;
         }
-        String files[] = file.list();
+        String files[] = dir.list();
 
         // Figure out which files are sessions
         int keycount = 0;
-        for (int i = 0; i < files.length; i++) {
-            if (files[i].endsWith(FILE_EXT)) {
-                keycount++;
+        if (files != null) {
+            for (String file : files) {
+                if (file.endsWith(FILE_EXT)) {
+                    keycount++;
+                }
             }
         }
-        return (keycount);
-
+        return keycount;
     }
 
 
     // --------------------------------------------------------- Public Methods
 
-
     /**
      * Remove all of the Sessions in this Store.
      *
      * @exception IOException if an input/output error occurs
      */
     @Override
-    public void clear()
-        throws IOException {
-
+    public void clear() throws IOException {
         String[] keys = keys();
-        for (int i = 0; i < keys.length; i++) {
-            remove(keys[i]);
+        for (String key : keys) {
+            remove(key);
         }
-
     }
 
 
@@ -204,30 +177,27 @@ public final class FileStore extends StoreBase {
      */
     @Override
     public String[] keys() throws IOException {
-
         // Acquire the list of files in our storage directory
-        File file = directory();
-        if (file == null) {
-            return (new String[0]);
+        File dir = directory();
+        if (dir == null) {
+            return new String[0];
         }
+        String files[] = dir.list();
 
-        String files[] = file.list();
-        
         // Bugzilla 32130
-        if((files == null) || (files.length < 1)) {
-            return (new String[0]);
+        if (files == null || files.length < 1) {
+            return new String[0];
         }
 
         // Build and return the list of session identifiers
-        ArrayList<String> list = new ArrayList<String>();
+        List<String> list = new ArrayList<>();
         int n = FILE_EXT.length();
-        for (int i = 0; i < files.length; i++) {
-            if (files[i].endsWith(FILE_EXT)) {
-                list.add(files[i].substring(0, files[i].length() - n));
+        for (String file : files) {
+            if (file.endsWith(FILE_EXT)) {
+                list.add (file.substring(0, file.length() - n));
             }
         }
-        return list.toArray(new String[list.size()]);
-
+        return list.toArray(new String[0]);
     }
 
 
@@ -242,79 +212,36 @@ public final class FileStore extends StoreBase {
      * @exception IOException if an input/output error occurs
      */
     @Override
-    public Session load(String id)
-        throws ClassNotFoundException, IOException {
-
+    public Session load(String id) throws ClassNotFoundException, IOException {
         // Open an input stream to the specified pathname, if any
         File file = file(id);
-        if (file == null) {
-            return (null);
+        if (file == null || !file.exists()) {
+            return null;
         }
 
-        if (! file.exists()) {
-            return (null);
-        }
-        if (manager.getContainer().getLogger().isDebugEnabled()) {
-            manager.getContainer().getLogger().debug(sm.getString(getStoreName()+".loading",
-                             id, file.getAbsolutePath()));
+        Context context = getManager().getContext();
+        Log contextLog = context.getLogger();
+
+        if (contextLog.isDebugEnabled()) {
+            contextLog.debug(sm.getString(getStoreName()+".loading", id, file.getAbsolutePath()));
         }
 
-        FileInputStream fis = null;
-        BufferedInputStream bis = null;
-        ObjectInputStream ois = null;
-        Loader loader = null;
-        ClassLoader classLoader = null;
-        ClassLoader oldThreadContextCL = Thread.currentThread().getContextClassLoader();
-        try {
-            fis = new FileInputStream(file.getAbsolutePath());
-            bis = new BufferedInputStream(fis);
-            Container container = manager.getContainer();
-            if (container != null)
-                loader = container.getLoader();
-            if (loader != null)
-                classLoader = loader.getClassLoader();
-            if (classLoader != null) {
-                Thread.currentThread().setContextClassLoader(classLoader);
-                ois = new CustomObjectInputStream(bis, classLoader);
-            } else {
-                ois = new ObjectInputStream(bis);
-            }
+        ClassLoader oldThreadContextCL = context.bind(Globals.IS_SECURITY_ENABLED, null);
 
-            StandardSession session =
-                    (StandardSession) manager.createEmptySession();
+        try (FileInputStream fis = new FileInputStream(file.getAbsolutePath());
+                ObjectInputStream ois = getObjectInputStream(fis)) {
+
+            StandardSession session = (StandardSession) manager.createEmptySession();
             session.readObjectData(ois);
             session.setManager(manager);
-            return (session);
+            return session;
         } catch (FileNotFoundException e) {
-            if (manager.getContainer().getLogger().isDebugEnabled())
-                manager.getContainer().getLogger().debug("No persisted data file found");
-            return (null);
-        } catch (IOException e) {
-            if (bis != null) {
-                try {
-                    bis.close();
-                } catch (IOException f) {
-                    // Ignore
-                }
+            if (contextLog.isDebugEnabled()) {
+                contextLog.debug("No persisted data file found");
             }
-            if (fis != null) {
-                try {
-                    fis.close();
-                } catch (IOException f) {
-                    // Ignore
-                }
-            }
-            throw e;
+            return null;
         } finally {
-            if (ois != null) {
-                // Close the input stream
-                try {
-                    ois.close();
-                } catch (IOException f) {
-                    // Ignore
-                }
-            }
-            Thread.currentThread().setContextClassLoader(oldThreadContextCL);
+            context.unbind(Globals.IS_SECURITY_ENABLED, oldThreadContextCL);
         }
     }
 
@@ -330,17 +257,18 @@ public final class FileStore extends StoreBase {
      */
     @Override
     public void remove(String id) throws IOException {
-
         File file = file(id);
         if (file == null) {
             return;
         }
-        if (manager.getContainer().getLogger().isDebugEnabled()) {
-            manager.getContainer().getLogger().debug(sm.getString(getStoreName()+".removing",
+        if (manager.getContext().getLogger().isDebugEnabled()) {
+            manager.getContext().getLogger().debug(sm.getString(getStoreName() + ".removing",
                              id, file.getAbsolutePath()));
         }
-        file.delete();
 
+        if (file.exists() && !file.delete()) {
+            throw new IOException(sm.getString("fileStore.deleteSessionFailed", file));
+        }
     }
 
 
@@ -354,85 +282,55 @@ public final class FileStore extends StoreBase {
      */
     @Override
     public void save(Session session) throws IOException {
-
         // Open an output stream to the specified pathname, if any
         File file = file(session.getIdInternal());
         if (file == null) {
             return;
         }
-        if (manager.getContainer().getLogger().isDebugEnabled()) {
-            manager.getContainer().getLogger().debug(sm.getString(getStoreName()+".saving",
+        if (manager.getContext().getLogger().isDebugEnabled()) {
+            manager.getContext().getLogger().debug(sm.getString(getStoreName() + ".saving",
                              session.getIdInternal(), file.getAbsolutePath()));
         }
-        FileOutputStream fos = null;
-        ObjectOutputStream oos = null;
-        try {
-            fos = new FileOutputStream(file.getAbsolutePath());
-            oos = new ObjectOutputStream(new BufferedOutputStream(fos));
-        } catch (IOException e) {
-            if (fos != null) {
-                try {
-                    fos.close();
-                } catch (IOException f) {
-                    // Ignore
-                }
-            }
-            throw e;
-        }
 
-        try {
+        try (FileOutputStream fos = new FileOutputStream(file.getAbsolutePath());
+                ObjectOutputStream oos = new ObjectOutputStream(new BufferedOutputStream(fos))) {
             ((StandardSession)session).writeObjectData(oos);
-        } finally {
-            oos.close();
         }
-
     }
 
 
     // -------------------------------------------------------- Private Methods
 
-
     /**
      * Return a File object representing the pathname to our
      * session persistence directory, if any.  The directory will be
      * created if it does not already exist.
      */
     private File directory() throws IOException {
-
         if (this.directory == null) {
-            return (null);
+            return null;
         }
         if (this.directoryFile != null) {
             // NOTE:  Race condition is harmless, so do not synchronize
-            return (this.directoryFile);
+            return this.directoryFile;
         }
         File file = new File(this.directory);
         if (!file.isAbsolute()) {
-            Container container = manager.getContainer();
-            if (container instanceof Context) {
-                ServletContext servletContext =
-                    ((Context) container).getServletContext();
-                File work = (File)
-                    servletContext.getAttribute(ServletContext.TEMPDIR);
-                file = new File(work, this.directory);
-            } else {
-                throw new IllegalArgumentException
-                    ("Parent Container is not a Context");
-            }
+            Context context = manager.getContext();
+            ServletContext servletContext = context.getServletContext();
+            File work = (File) servletContext.getAttribute(ServletContext.TEMPDIR);
+            file = new File(work, this.directory);
         }
         if (!file.exists() || !file.isDirectory()) {
             if (!file.delete() && file.exists()) {
-                throw new IOException(
-                        sm.getString("fileStore.deleteFailed", file));
+                throw new IOException(sm.getString("fileStore.deleteFailed", file));
             }
             if (!file.mkdirs() && !file.isDirectory()) {
-                throw new IOException(
-                        sm.getString("fileStore.createFailed", file));
+                throw new IOException(sm.getString("fileStore.createFailed", file));
             }
         }
         this.directoryFile = file;
-        return (file);
-
+        return file;
     }
 
 
@@ -444,15 +342,20 @@ public final class FileStore extends StoreBase {
      *    used in the file naming.
      */
     private File file(String id) throws IOException {
-
-        if (this.directory == null) {
-            return (null);
+        File storageDir = directory();
+        if (storageDir == null) {
+            return null;
         }
+
         String filename = id + FILE_EXT;
-        File file = new File(directory(), filename);
-        return (file);
+        File file = new File(storageDir, filename);
 
+        // Check the file is within the storage directory
+        if (!file.getCanonicalFile().toPath().startsWith(storageDir.getCanonicalFile().toPath())) {
+            log.warn(sm.getString("fileStore.invalid", file.getPath(), id));
+            return null;
+        }
+
+        return file;
     }
-
-
 }
