@@ -274,8 +274,9 @@ public class Cookie {
             skipResult = skipByte(bb, COMMA_BYTE);
             if (skipResult == SkipResult.FOUND) {
                 parseAttributes = false;
+            } else {
+                skipResult = skipByte(bb, SEMICOLON_BYTE);
             }
-            skipResult = skipByte(bb, SEMICOLON_BYTE);
             if (skipResult == SkipResult.EOF) {
                 parseAttributes = false;
                 moreToProcess = false;
@@ -285,6 +286,7 @@ public class Cookie {
             }
 
             if (parseAttributes) {
+                skipLWS(bb);
                 skipResult = skipBytes(bb, PATH_BYTES);
                 if (skipResult == SkipResult.FOUND) {
                     skipLWS(bb);
@@ -293,6 +295,7 @@ public class Cookie {
                         skipInvalidCookie(bb);
                         continue;
                     }
+                    skipLWS(bb);
                     path = readCookieValueRfc2109(bb, true);
                     if (path == null) {
                         skipInvalidCookie(bb);
@@ -303,8 +306,9 @@ public class Cookie {
                     skipResult = skipByte(bb, COMMA_BYTE);
                     if (skipResult == SkipResult.FOUND) {
                         parseAttributes = false;
+                    } else {
+                        skipResult = skipByte(bb, SEMICOLON_BYTE);
                     }
-                    skipResult = skipByte(bb, SEMICOLON_BYTE);
                     if (skipResult == SkipResult.EOF) {
                         parseAttributes = false;
                         moreToProcess = false;
@@ -316,6 +320,7 @@ public class Cookie {
             }
 
             if (parseAttributes) {
+                skipLWS(bb);
                 skipResult = skipBytes(bb, DOMAIN_BYTES);
                 if (skipResult == SkipResult.FOUND) {
                     skipLWS(bb);
@@ -324,17 +329,20 @@ public class Cookie {
                         skipInvalidCookie(bb);
                         continue;
                     }
+                    skipLWS(bb);
                     domain = readCookieValueRfc2109(bb, false);
                     if (domain == null) {
                         skipInvalidCookie(bb);
                         continue;
                     }
+                    skipLWS(bb);
 
                     skipResult = skipByte(bb, COMMA_BYTE);
                     if (skipResult == SkipResult.FOUND) {
                         parseAttributes = false;
+                    } else {
+                        skipResult = skipByte(bb, SEMICOLON_BYTE);
                     }
-                    skipResult = skipByte(bb, SEMICOLON_BYTE);
                     if (skipResult == SkipResult.EOF) {
                         parseAttributes = false;
                         moreToProcess = false;
@@ -415,12 +423,12 @@ public class Cookie {
     private static SkipResult skipBytes(ByteBuffer bb, byte[] target) {
         int mark = bb.position();
 
-        for (int i = 0; i < target.length; i++) {
+        for (byte b : target) {
             if (!bb.hasRemaining()) {
                 bb.position(mark);
                 return SkipResult.EOF;
             }
-            if (bb.get() != target[i]) {
+            if (bb.get() != b) {
                 bb.position(mark);
                 return SkipResult.NOT_FOUND;
             }
