@@ -18,10 +18,8 @@ package org.apache.catalina.tribes.group.interceptors;
 
 import java.util.ArrayList;
 
-import static org.junit.Assert.assertEquals;
-import static org.junit.Assert.fail;
-
 import org.junit.After;
+import org.junit.Assert;
 import org.junit.Before;
 import org.junit.Test;
 
@@ -31,18 +29,10 @@ import org.apache.catalina.tribes.ChannelException;
 import org.apache.catalina.tribes.ManagedChannel;
 import org.apache.catalina.tribes.Member;
 import org.apache.catalina.tribes.MembershipListener;
+import org.apache.catalina.tribes.TesterUtil;
 import org.apache.catalina.tribes.group.GroupChannel;
+import org.apache.catalina.tribes.transport.ReceiverBase;
 
-/**
- * <p>Title: </p>
- *
- * <p>Description: </p>
- *
- * <p>Company: </p>
- *
- * @author not attributable
- * @version 1.0
- */
 public class TestTcpFailureDetector {
     private TcpFailureDetector tcpFailureDetector1 = null;
     private TcpFailureDetector tcpFailureDetector2 = null;
@@ -55,6 +45,8 @@ public class TestTcpFailureDetector {
     public void setUp() throws Exception {
         channel1 = new GroupChannel();
         channel2 = new GroupChannel();
+        ((ReceiverBase) channel1.getChannelReceiver()).setHost("localhost");
+        ((ReceiverBase) channel2.getChannelReceiver()).setHost("localhost");
         channel1.getMembershipService().setPayload("Channel-1".getBytes("ASCII"));
         channel2.getMembershipService().setPayload("Channel-2".getBytes("ASCII"));
         mbrlist1 = new TestMbrListener("Channel-1");
@@ -65,6 +57,7 @@ public class TestTcpFailureDetector {
         channel2.addInterceptor(tcpFailureDetector2);
         channel1.addMembershipListener(mbrlist1);
         channel2.addMembershipListener(mbrlist2);
+        TesterUtil.addRandomDomain(new ManagedChannel[] {channel1, channel2});
     }
 
     public void clear() {
@@ -79,16 +72,25 @@ public class TestTcpFailureDetector {
         channel1.start(Channel.DEFAULT);
         channel2.start(Channel.DEFAULT);
         //Thread.sleep(1000);
-        assertEquals("Expecting member count to be equal",mbrlist1.members.size(),mbrlist2.members.size());
+        Assert.assertEquals("Expecting member count to be equal",mbrlist1.members.size(),mbrlist2.members.size());
         channel2.stop(Channel.SND_RX_SEQ);
         ByteMessage msg = new ByteMessage(new byte[1024]);
         try {
-            channel1.send(channel1.getMembers(), msg, 0);
-            fail("Message send should have failed.");
+            int msgCount = 0;
+            // Normally the first message sent should fail but occasional
+            // failures are observed on CI systems so messages are sent in a
+            // loop with a delay between them to try and account for timing
+            // differences.
+            while (msgCount < 5) {
+                channel1.send(channel1.getMembers(), msg, 0);
+                msgCount++;
+                Thread.sleep(500);
+            }
+            Assert.fail("Message send should have failed.");
         } catch ( ChannelException x ) {
             // Ignore
         }
-        assertEquals("Expecting member count to not be equal",mbrlist1.members.size()+1,mbrlist2.members.size());
+        Assert.assertEquals("Expecting member count to not be equal",mbrlist1.members.size()+1,mbrlist2.members.size());
         channel1.stop(Channel.DEFAULT);
         channel2.stop(Channel.DEFAULT);
     }
@@ -104,7 +106,7 @@ public class TestTcpFailureDetector {
         channel2.stop(Channel.SND_RX_SEQ);
         channel2.start(Channel.MBR_TX_SEQ);
         //Thread.sleep(1000);
-        assertEquals("Expecting member count to not be equal",mbrlist1.members.size()+1,mbrlist2.members.size());
+        Assert.assertEquals("Expecting member count to not be equal",mbrlist1.members.size()+1,mbrlist2.members.size());
         channel1.stop(Channel.DEFAULT);
         channel2.stop(Channel.DEFAULT);
     }
@@ -116,15 +118,15 @@ public class TestTcpFailureDetector {
         channel1.start(Channel.DEFAULT);
         channel2.start(Channel.DEFAULT);
         //Thread.sleep(1000);
-        assertEquals("Expecting member count to be equal",mbrlist1.members.size(),mbrlist2.members.size());
+        Assert.assertEquals("Expecting member count to be equal",mbrlist1.members.size(),mbrlist2.members.size());
         channel2.stop(Channel.MBR_TX_SEQ);
         ByteMessage msg = new ByteMessage(new byte[1024]);
         try {
             Thread.sleep(5000);
-            assertEquals("Expecting member count to be equal",mbrlist1.members.size(),mbrlist2.members.size());
+            Assert.assertEquals("Expecting member count to be equal",mbrlist1.members.size(),mbrlist2.members.size());
             channel1.send(channel1.getMembers(), msg, 0);
         } catch ( ChannelException x ) {
-            fail("Message send should have succeeded.");
+            Assert.fail("Message send should have succeeded.");
         }
         channel1.stop(Channel.DEFAULT);
         channel2.stop(Channel.DEFAULT);
@@ -134,9 +136,17 @@ public class TestTcpFailureDetector {
     public void tearDown() throws Exception {
         tcpFailureDetector1 = null;
         tcpFailureDetector2 = null;
-        try { channel1.stop(Channel.DEFAULT);}catch (Exception ignore){ /* Ignore */ }
+        try {
+            channel1.stop(Channel.DEFAULT);
+        } catch (Exception ignore) {
+            // Ignore
+        }
         channel1 = null;
-        try { channel2.stop(Channel.DEFAULT);}catch (Exception ignore){ /* Ignore */ }
+        try {
+            channel2.stop(Channel.DEFAULT);
+        } catch (Exception ignore) {
+            // Ignore
+        }
         channel2 = null;
     }
 
@@ -145,7 +155,7 @@ public class TestTcpFailureDetector {
         public TestMbrListener(String name) {
             this.name = name;
         }
-        public ArrayList<Member> members = new ArrayList<Member>();
+        public ArrayList<Member> members = new ArrayList<>();
         @Override
         public void memberAdded(Member member) {
             if ( !members.contains(member) ) {
